/**
* Returns whether this object is equal to the specified _object.
* @param {Object} _object
* @return {Boolean}
*/
Boolean.prototype.equals = function(_object) {

	if(this === _object) {

		return true;
	}

	if(!typeof(_object) == "boolean") {

		return false;
	}

	return this.toString() == _object.toString();
};
/**
* Round a Number to the specified number of decimals.
* @name toFixed
* @function
* @memberOf Number
*/
/** @ignore */
Number.prototype._$toFixed$_ = function(){throw "don't call me, used for reflection.";}

/**
* Convert the value of the object into an exponential notation.
* @name toExponential
* @function
* @memberOf Number
*/
/** @ignore */
Number.prototype._$toExponential$_ = function(){throw "don't call me, used for reflection.";}

/**
* Converts the value of the object into an exponential notation if it has more digits than specified.
* @name toPrecision
* @function
* @memberOf Number
*/
/** @ignore */
Number.prototype._$toPrecision$_ = function(){throw "don't call me, used for reflection.";}

/**
* Executes the block the same number of times the number represents. Parameter to the block is an number.
* @param {Function} _block
*/
Number.prototype.times = function(_block) {

	for(var i=0;i<this;i++) {

		_block(i);
	}
}

/**
* Executes the block the number of times upto the other number. Parameter to the block is an number.
* @param {Number} _other
* @param {Function} _block
*/
Number.prototype.upto = function(_other, _block) {

	for(var i=this;i<=_other;i++) {

		_block(i);
	}
}

/**
* Executes the block the number of times downto the other number. Parameter to the block is an number.
* @param {Number} _other
* @param {Function} _block
*/	
Number.prototype.downto = function(_other, _block) {

	for(var i=this;i>=_other;i--){

		_block(i);
	}
}

/**
* Executes the block. Step using the other number. Parameter to the block is an number.
* @param {Number} _other
* @param {Function} _block
*/
Number.prototype.towards = function(_other, _block) {

	var step = this.compare(_other);
	for (var i=this;i!==(_other-step);i -= step) {

		_block(i);
	}
}

/**
* Returns the number incremented by 1.
* @return {Number}
*/
Number.prototype.succ = function() {

	return this + 1;
}

/**
* Returns the number decremented by 1.
* @return {Number}
*/
Number.prototype.pred = function() {

	return this - 1;
}

/**
* Returns the character code this number represents.
* @return {Number}
*/
Number.prototype.toChar = function() {

	return String.fromCharCode(this);
}

/**
* Compares the number against the other number. Returns -1, 0, 1.
* @param {Number} _other
* @return {Number}
*/
Number.prototype.compare = function(_other) {

	if(this < _other){ return -1; }
	if(this > _other){ return +1; }
	return 0;
}

/**
* Returns a new array of numbers from the current number to the specified number.
* @param {Number} _other Inclusive
* @return {Array}
*/
Number.prototype.toArray = function(_other) {

	var numbers = [];
	this.upto(_other, function(_value){numbers[numbers.length]=_value;});
	return numbers;
}

/**
* Returns whether this object is equal to the specified _object.
* @param {Object} _object
* @return {Boolean}
*/
Number.prototype.equals = function(_object) {

	if(this === _object){

		return true;
	}

	if(!typeof(_object) == "number") {

		return false;
	}

	return this.toString() == _object.toString();
}

/**
* Largest number that is less than infinity.
* @name MAX_VALUE
*/
Number._$MAX_VALUE$_ = "don't use me, used for reflection.";
/**
* Smallest number that is greater than negative infinity.
* @name MIN_VALUE
*/
Number._$MIN_VALUE$_ = "don't use me, used for reflection.";
/**
* Not a number.
* @name NaN
*/
Number._$NaN$_ = "don't use me, used for reflection.";
/**
* Out of range negative number.
* @name NEGATIVE_INFINITY
*/
Number._$NEGATIVE_INFINITY$_ = "don't use me, used for reflection.";
/**
* Out of range positive number.
* @name POSITIVE_INFINITY
*/
Number._$POSITIVE_INFINITY$_ = "don't use me, used for reflection.";
/**
* Out of range number.
* @name INFINITY
*/
Number.INFINITY = Infinity;
/**
* Calls parseFloat native on the global object.
*/
Number.parseFloat = function(_number){return $GLOBAL_OBJECT.parseFloat(_number);};
/**
* Calls parseInt native on the global object.
*/
Number.parseInt = function(_number, radix){return $GLOBAL_OBJECT.parseInt(_number, radix);};
/**
* Calls isFinite native on the global object.
*/
Number.isFinite = function(_number){return $GLOBAL_OBJECT.isFinite(_number);};
/**
* Calls isNaN native on the global object.
*/
Number.isNaN = function(_number){return $GLOBAL_OBJECT.isNaN(_number);};
/**
* @class Enumerable
* @description <code>Enumerable</code> is a Mixin class. It provides a set of helper methods for any class that implements the each method.
* @example
<pre>
	class MyEnumerable mixin Enumerable
</pre>
*/
var Enumerable = function(){this.init.apply(this, arguments);};
exports.Enumerable = Enumerable;

/**
* Use to break out of a loop within closure.
* @example
<div>var result = 0;</div>
<div>var array = [1, 2, 3, 4, 5];</div>
<div>array.each(function(_value)</div>
<div>{</div>
<div>	if(_value == 4)</div>
<div>	{</div>
<div>		array.$break();</div>
<div>	}</div>
<div>	result = _value;</div>
<div>});</div>
<div>this.assertEquals(3, result);</div>
*/
Enumerable.$break = function(){throw Enumerable.$break;};
/**
* Use to continue a loop within closure.
* @example
<div>var result = 0;</div>
<div>var array = [1, 2, 3, 4, 5];</div>
<div>array.each(function(_value)</div>
<div>{</div>
<div>	if(_value != 4)</div>
<div>	{</div>
<div>		array.$continue();</div>
<div>	}</div>
<div>	result = _value;</div>
<div>});</div>
<div>this.assertEquals(4, result);</div>
*/
Enumerable.$continue = function(){throw Enumerable.$continue;};

/**
* Returns the same object as the specified object.
* @param {Object} _object
* @return {Object}
*/
Enumerable.echo = function(_object)
{
	return _object;
};

/**
* Returns an array of items from the specified iterable.
* @param {Object} _iterable If _iterable is null or undefined an empty array is returned. If _iterable.toArray exists, it will be used
*  to convert. If _iterable.length exists, it is used to convert.
* @return {Array}
*/
Enumerable.convertToArray = function(_iterable)
{
	if(!_iterable){ return []; }
	if(_iterable.toArray)
	{
		return _iterable.toArray();

	} else {
		var results = [];
		for (var i=0;i<_iterable.length;i++)
		{
			results[results.length] = _iterable[i];
		}
		return results;
	}
};

/**
* Passes each element of a enumeration to the specifed block. Returns true if the block nevers returns false.
* @example
<div>["ant", "bear", "cat"].all(function(_value){return _value.length >= 3}); -> true</div>
<div>["ant", "bear", "cat"].all(function(_value){return _value.length >= 4}); -> false</div>
<div>[null, Object.NOT_DEF,,"", 99].all(); -> false</div>
* @param {Function} _block If the given block is null, Enumerable#echo is used.
* @return {Boolean}
*/
Enumerable.all = function(_block)
{
	var me = this;
	var result = true;
	this.each(function(_object1, _object2)
	{
		result = result && !!(_block || me.echo)(_object1, _object2);
		if(!result)
		{
			me.$break();
		}
	});
	return result;
};

/**
* Passes each element of a enumeration to the specifed block. Returns true if the block returns at least one true.
* @example
<div>["ant", "bear", "cat"].any(function(_value){return _value.length >= 3}); -> true</div>
<div>["ant", "bear", "cat"].any(function(_value){return _value.length >= 4}); -> true</div>
<div>[null, Object.NOT_DEF,,"", 99].any(); -> true</div>
* @param {Function} _block If the given block is null, Enumerable#echo is used.
* @return {Boolean}
*/
Enumerable.any = function(_block)
{
	var me = this;
	var result = true;
	this.each(function(_object1, _object2)
	{
		result = !!(_block || me.echo)(_object1, _object2);
		if(result)
		{
			me.$break();
		}
	});
	return result;
};

/**
* Returns a new array of results from the specifed block.
* @example
<div>[1, 2, 3, 4].collect(function(_value){return _value*_value}); -> [1, 4, 9, 16]</div>
<div>[1, 2, 3, 4].collect(function(_value){return "cat"}); -> ["cat", "cat", "cat", "cat"]</div>
* @param {Function} _block
* @return {Array}
*/
Enumerable.collect = function(_block)
{
	var me = this;
	var results = [];
	this.each(function(_object1, _object2)
	{
		results[results.length]	= (_block || me.echo)(_object1, _object2);
	});
	return results;
};

/**
* Passes each element in the enumeration to the specified block. Returns the first for which block is not false.
* @example
<div>Number(1).toArray(10).detect(function(_value){return _value % 5 == 0 && _value % 7 == 0}); -> undefined</div>
<div>Number(1).toArray(100).detect(function(_value){return _value % 5 == 0 && _value % 7 == 0}); -> 35</div>
* @param {Function} _block
* @return {Object}
*/
Enumerable.detect = function(_block)
{
	var me = this;
	var result;
	this.each(function(_object1, _object2)
	{
		if(_block(_object1, _object2))
		{
			result = _object1;
			me.$break();
		}
	});
	return result;
};

/**
* Passes each element in the enumeration to the specified block. Returns a new array of elements where the block returns true.
* @example
Number(1).toArray(10).findAll(function(_value){return _value % 3 == 0}); -> [3, 6, 9]
* @param {Function} _block
* @return {Array}
*/
Enumerable.findAll = function(_block)
{
	var results = [];
	this.each(function(_object1, _object2)
	{
		if(_block(_object1, _object2))
		{
			results[results.length] = _object1;
		}
	});
	return results;
};

/**
* Passes each element in the enumeration to the specified block. The specified regexp pattern is used to test against the first object's <code>toString</code>.
*  If the test is true, the specified block is called with parameters from the <code>each</code> method, and the results from the block
*  are added to the array.
* @example
<div>["Matthew", "Mathew", "Matt", "Mat"].grep(/Mat/); -> ["Matthew", "Mathew", "Matt", "Mat"]</div>
<div>["Matthew", "Mathew", "Matt", "Mat"].grep(/Matt/); -> ["Matthew", "Matt"]</div>
<div>[1, 2, 3, 4, 5].grep(/3|4/); -> [3, 4]</div>
* @param {RegExp} _pattern
* @param {Function} _block
* @return {Array}
*/
Enumerable.grep = function(_pattern, _block)
{
	var me = this;
	var results = [];
	this.each(function(_object1, _object2)
	{
		var stringValue = _object1.toString();
		if(stringValue.match(_pattern))
		{
			results[results.length] = (_block || me.echo)(_object1, _object2);
		}
	});
	return results;
};

/**
* Returns whether the specified object exists in the enumerable.
* @example
<div>["Matthew", "Mathew", "Matt", "Mat"].include("Mat"); -> true</div>
<div>["Matthew", "Mathew", "Matt", "Mat"].include("Matt"); -> true</div>
<div>["Matthew", "Mathew", "Matt", "Mat"].include("Sam"); -> false</div>
<div>[1, 2, 3, 4, 5].include(3); -> true</div>
* @param {Object} _object
* @return {Boolean}
*/
Enumerable.include = function(_object)
{
	var me = this;
	var found = false;
	this.each(function(_object1)
	{
		if(_object1 && !_object1.equals){ _object1.equals = function(_test){ return this == _test; }; }

		if(_object1 != null && _object1.equals(_object))
		{
			found = true;
			me.$break();
		}
	});
	return found;
};

/**
* Combines the elements of enumeration by applying the block to an accumulator value (memo) and each element in turn.
*  At each step, memo is set to the value returned by the block.
* @example
<div>[5, 6, 7, 8, 9, 10].inject(0, function(_memo, _value){return _memo + _value}); -> 45</div>
<div>[5, 6, 7, 8, 9, 10].inject(1, function(_memo, _value){return _memo * _value}); -> 151200</div>
<div>["cat", "sheep", "bear"].inject("", function(_memo, _value){return _memo.length > _value.length ? _memo : _value}); -> "sheep"</div>
<div>["cat", "sheep", "bear"].inject(0, function(_memo, _value){return _memo >= _value.length ? _memo : _value.length}); -> 5</div>
* @param {Object} _memo
* @param {Function} _block
* @return {Object}
*/
Enumerable.inject = function(_memo, _block)
{
	this.each(function(_object1, _object2)
	{
		_memo = _block(_memo, _object1, _object2);
	});
	return _memo;
};

/**
* Invokes the specified method for each element in the enumeration. Returns an array of results from the invocation of the specified method.
* @example
[[1], [1, 2], [1, 2, 3], [1, 2, 3, 4], [1, 2, 3, 4, 5]].invoke("join", "-"); -> ["1", "1-2", "1-2-3", "1-2-3-4", "1-2-3-4-5"]
* @param {String} _method The name of the method to call on the object. Any parameter after the method name will act as parameters to the method call.
* @return {Array}
*/
Enumerable.invoke = function(_method)
{
	var args = this.convertToArray(arguments).slice(1);
	return this.collect(function(_object1)
	{
		return _object1[_method].apply(_object1, args);
	});
};

/**
* Returns the max element in the enumeration using the specifed block to return a comparable value -1, 0, 1.
* @example
<div>["albatross", "dog", "horse"].max(); -> "horse"</div>
<div>["albatross", "dog", "horse"].max(function(a, _index, b){return a.length<b.length?-1:a.length>b.length?1:0}); -> "albatross"</div>
* @param {Function} _block
* @return {Object}
*/
Enumerable.max = function(_block)
{
	var me = this;
	var result;
	this.each(function(_object1, _object2)
	{
		if(!result)
		{
			result = _object1;

		} else {
			if(!_block)
			{
				if(_object1 >= result)
				{
					result = _object1;
				}

			} else {

				var comp = _block(_object1, _object2, result);
				if(comp >= 0)
				{
					result = _object1;
				}
			}
		}
	});
	return result;
};

/**
* Returns the min element in the enumeration using the specifed block to return a comparable value -1, 0, 1.
* @example
<div>["albatross", "dog", "horse"].max(); -> "albatross"</div>
<div>["albatross", "dog", "horse"].min(function(a, _index, b){return a.length<b.length?-1:a.length>b.length?1:0}); -> "dog"</div>
* @param {Function} _block
* @return {Object}
*/
Enumerable.min = function(_block)
{
	var me = this;
	var result;
	this.each(function(_object1, _object2)
	{
		if(!result)
		{
			result = _object1;

		} else {

			if(!_block)
			{
				if(_object1 < result)
				{
					result = _object1;
				}

			} else {

				var comp = _block(_object1, _object2, result);
				if(comp < 0)
				{
					result = _object1;
				}
			}
		}
	});
	return result;
};

/**
* Uses the specified block to result in true or false and split the results.
* @example
[1, 2, 3, 4, 5, 6].partition(function(_value){ return (_value%2 == 0) }); -> [[2,4,6],[1,3,5]]
* @param {Function} _block
* @return {Array} [0] = true objects, [1] = false objects
*/
Enumerable.partition = function(_block)
{
	var me = this;
	var trues = [];
	var falses = [];
	this.each(function(_object1, _object2)
	{
		if((_block || me.echo)(_object1, _object2))
		{
			trues[trues.length] = _object1;

		} else {

			falses[falses.length] = _object1;
		}
	});
	return [trues, falses];
};

/**
* Passes the specified property to each element in the enumeration and using the first object to evaluate the property. Returns a new
*  array element containing the value of object[property].
* @example
[[1], [1,2], [1,2,3], [1,2,3,4]].pluck("length"); -> [1,2,3,4]
* @param {String} _property
* @return {Array}
*/
Enumerable.pluck = function(_property)
{
	var results = [];
	this.each(function(_object1, _object2)
	{
		results[results.length] = _object1[_property];
	});
	return results;
};

/**
* Returns a new array where the specifed block results in false.
* @example
Number(1).toArray(10).reject(function(_value){return (_value%3==0)}); -> [1, 2, 4, 5, 7, 8, 10]
* @param {Function} _block
* @return {Array}
*/
Enumerable.reject = function(_block)
{
	var results = [];
	this.each(function(_object1, _object2)
	{
		if(!_block(_object1, _object2))
		{
			results[results.length] = _object1;
		}
	});
	return results;
};

/**
* Returns a new array using the specified block to return a comparable value.
* @example
<div>["rhea", "kea", "flea"].sortBy(); -> ["flea", "kea", "rhea"]</div>
<div>Number(1).toArray(10).sortBy(function(_value, _index){return -1 * _value}); -> [10, 9, 8, 7, 6, 5, 4, 3, 2, 1]</div>
<div>["apple", "pear", "fig"].sortBy(function(_value, _index){return _value.length}); -> ["fig", "pear", "apple"]</div>
* @param {Function} _block
* @return {Array}
*/
Enumerable.sortBy = function(_block)
{
	var me = this;
	var items = this.collect(function(_object1, _object2)
				{
					return {value:_object1, criteria:(_block || me.echo)(_object1, _object2)};
				});

	items.sort(function(left, right)
	{
		var a = left.criteria;
		var b = right.criteria;
		return a < b ? -1 : a > b ? 1 : 0;
	});

	return items.pluck("value");
};

/**
* Converts any arguments to arrays, then merges elements of enum with corresponding elements from each argument.
*  This generates a sequence of enum#size n-element arrays, where n is one more that the count of arguments.
*  If the size of any argument is less than enum#size, nil values are supplied. If a block given, it is invoked for
*  each output array, otherwise an array of arrays is returned.
* @example
<div>[1, 2, 3].zip([4, 5, 6], [7, 8, 9]); -> [[1, 4, 7], [2, 5, 8], [3, 6, 9]]</div>
<div>["cat", "dog"].zip([1]); -> [["cat", 1], ["dog", null]]</div>
<div>[1, 2, 3].zip(); -> [[1], [2], [3]]</div>
* @param {Function} _block
* @return {Array}
*/
Enumerable.zip = function()
{
	var _block = this.echo;
	var args = this.convertToArray(arguments);

	// see if we have a transform block
	var last = args[args.length-1];
	if(typeof last == "function")
	{
		_block = args.pop();
	}

	var collections = [this].concat(args).map(this.convertToArray);

	return this.map(function(_object1, _object2)
					{
						return _block(collections.pluck(_object2));
					});
};

/**
* Calls Enumerable.collect.
*/
Enumerable.toArray = function(){return this.collect();};
/**
* Calls Enumerable.collect.
*/
Enumerable.entries = function(){return this.collect();};
/**
* Calls Enumerable.collect.
*/
Enumerable.map = function(_block){return this.collect(_block);};
/**
* Calls Enumerable.detect.
*/
Enumerable.find = function(_block){return this.detect(_block);};
/**
* Calls Enumerable.include.
*/
Enumerable.member = function(_object){return this.include(_object);};
/**
* Calls Enumerable.findAll.
*/
Enumerable.select = function(_block){return this.findAll(_block);};
/**
* @fileOverview Array is one of the core classes to native JavaScript. Array mixes in the Enumerable class
*  which gives Array a lot more functionality. See Enumerable to see all the additional methods on Array.
* @example
*/

/**
* Sets or returns the number of elements in an array.
* @name length
* @type {Number}
*/
/** @ignore */
Array.prototype._$length$_ = "don't use me, used for reflection.";

/**
* Joins two or more arrays and returns the result.
* @example
["Tom", "Dick", "Harry"].concat(["Matt", "Annie", "Sam"]); -> ["Tom", "Dick", "Harry", "Matt", "Annie", "Sam"]
* @name concat
* @param {Array} _args One or more array objects to be joined to an array. Array#unshift(_args1, _args2,.....,_argsX) allowed.
* @return {Array}
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$concat$_ = function(_args){throw "don't call me, used for reflection.";}

/**
* Puts all the elements of an array into a string. The elements are separated by a specified delimiter.
* @example
["Tom", "Dick", "Harry"].join(" - "); -> "Tom - Dick - Harry"
* @name join
* @param {String} [_sep] Specifies the separator to be used. Comma is the default separator.
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$join$_ = function(_sep){throw "don't call me, used for reflection.";}

/**
* Removes and returns the last element of an array.
* @example
["Tom", "Dick", "Harry"].pop(); -> "Harry"
* @name pop
* @return {Object}
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$pop$_ = function(){throw "don't call me, used for reflection.";}

/**
* Adds one or more elements to the end of an array and returns the new length.
* @example
["Tom", "Dick", "Harry"].push("Matt"); -> 4, contents = ["Tom", "Dick", "Harry", "Matt"]
* @name push
* @param {Object} _args One or more items. Array#push(_args1,.....,_argsX) allowed.
* @return {Number}
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$push$_ = function(_args){throw "don't call me, used for reflection.";}

/**
* Reverses the order of the elements in an array.
* @example
["Tom", "Dick", "Harry"].reverse(); -> ["Harry", "Dick", "Tom"]
* @name reverse
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$reverse$_ = function(){throw "don't call me, used for reflection.";}

/**
* Removes and returns the first element of an array.
* @example
["Tom", "Dick", "Harry"].shift(); -> "Tom", contents = ["Dick", "Harry"]
* @name shift
* @return {Object}
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$shift$_ = function(){throw "don't call me, used for reflection.";}

/**
* Returns selected elements from an existing array.
* @example
["Tom", "Dick", "Harry"].slice(1,3); -> ["Dick", "Harry"]
* @name slice
* @param {Number} _start
* @param {Number} [_end]
* @return {Array}
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$slice$_ = function(_start, _end){throw "don't call me, used for reflection.";}

/**
* Sorts the elements of an array.
* @example
["Tom", "Dick", "Harry"].sort(); -> ["Dick", "Harry", "Tom"]
* @name sort
* @param {Function} [_func] The block function of the sort implementation
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$sort$_ = function(_func){throw "don't call me, used for reflection.";}

/**
* Removes and adds new elements to an array.
* @example
<div>["Tom", "Dick", "Harry", "Matt", "Annie", "Sam"].splice(2, 0, "Carson"); -> contents = ["Tom", "Dick", "Carson", "Harry", "Matt", "Annie", "Sam"]</div>
<div>["Tom", "Dick", "Harry", "Matt", "Annie", "Sam"].splice(2, 1, "Carson"); -> contents = ["Tom", "Dick", "Carson", "Matt", "Annie", "Sam"]</div>
* @param {Number} _start Specify where to add/remove elements
* @param {Number} _deleteCount Specify how many elements should be removed. Greater then zero.
* @param {Object} [_args] Specify a new element to add to the array. Array#splice(_start, _deleteCount, _args1,.....,_argsX) allowed.
* @name splice
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$splice$_ = function(_start, _deleteCount, _args){throw "don't call me, used for reflection.";}

/**
* Adds one or more elements to the beginning of an array and returns the new length.
* @example
["Tom", "Dick", "Harry"].unshift("Matt", "Annie", "Sam"); -> 6, contents = ["Tom", "Dick", "Harry", "Matt", "Annie", "Sam"]
* @name unshift
* @param {Object} _args1 The first element to add to the array.
* @param {Object} [_args2] The second element to add to the array. Array#unshift(_args1, _args2,.....,_argsX) allowed.
* @function
* @memberOf Array
*/
/** @ignore */
Array.prototype._$unshift$_ = function(_args1, _args2){throw "don't call me, used for reflection.";}

/**
* Returns whether this object is equal to the specified _object.
* @example
["Tom", "Dick", "Harry"].equals(["Tom", "Dick", "Harry"]); -> true;
* @param {Object} _object
* @return {Boolean}
*/
Array.prototype.equals = function(_object) {

	if(this === _object) {

		return true;
	}
	if(this.constructor != _object.constructor) {

		return false;
	}

	// compare lengths
	if(this.size() != _object.size()){ return false; }

	// compare entries in the indexes
	for(var i=0;i<this.length;i++) {

		// if both element are null pass them.
		if((this[i] == null) && (_object[i] == null)){ continue; }

		if(!this[i].equals) {

			if(this[i] != _object[i]) {

				return false;
			}
		} else {

			if(!this[i].equals(_object[i])) {

				return false;
			}
		}
	}

	return true;
};

/**
* Returns a description of the array
* @return {String}
*/
Array.prototype.toString = function() {

	var me = this;
	var sb = "";
	sb += "[";

	this.each(function(_value, _index) {

		sb += _value.toString();
		if(_index != (me.size() -1)){ sb += ", "; }
	});

	sb += "]";
	return sb;
};

// enumerable methods
/**
* Required by the <code>Enumerable</code> mixin. Calls <code>Array#iterator(_block)</code> passing a value and index to the block.
* @param {Function} _block. Parameters passed to the _block are Object and the Number.
*/
Array.prototype.each = function(_block) {

	var me = this;
	var index = 0;
	try {

		this.iterator(function(value) {

			try {

				_block(value, index++);

			} catch(e) {

				if(e != Enumerable.$continue){ throw e; }
			}
		});

	} catch(e) {

		if(e != Enumerable.$break){ throw e; }
	}
}

/**
* Iterates through the array, executing the _block and passing a value from Array[index] as its parameter. Used by <code>Array#each(_block)</code>.
* @param {Function} _block. Parameter passed to the _block is the element object at the index.
*/
Array.prototype.iterator = function(_block) {

	for(var i=0;i<this.length;i++) {

		_block(this[i]);
	}
}

Array.prototype.$break = Enumerable.$break;
Array.prototype.$continue = Enumerable.$continue;
Array.prototype.echo = Enumerable.echo;
Array.prototype.convertToArray = Enumerable.convertToArray;
Array.prototype.all = Enumerable.all;
Array.prototype.any = Enumerable.any;
Array.prototype.collect = Enumerable.collect;
Array.prototype.detect = Enumerable.detect;
Array.prototype.findAll = Enumerable.findAll;
Array.prototype.grep = Enumerable.grep;
Array.prototype.include = Enumerable.include;
Array.prototype.inject = Enumerable.inject;
Array.prototype.invoke = Enumerable.invoke;
Array.prototype.max = Enumerable.max;
Array.prototype.min = Enumerable.min;
Array.prototype.partition = Enumerable.partition;
Array.prototype.pluck = Enumerable.pluck;
Array.prototype.reject = Enumerable.reject;
Array.prototype.sortBy = Enumerable.sortBy;
Array.prototype.zip = Enumerable.zip;
Array.prototype.toArray = Enumerable.toArray;
Array.prototype.entries = Enumerable.entries;
Array.prototype.map = Enumerable.map;
Array.prototype.find = Enumerable.find;
Array.prototype.member = Enumerable.member;
Array.prototype.select = Enumerable.select;

// querying methods
/**
* Returns the element at the specified position in this list.
* @param {Number} _index
* @return {Object}
*/
Array.prototype.get = function(_index) {

	return this[_index];
};

/**
* Returns the index in this array of the first occurence of the specified element, or -1 if the array does not contain this element.
* @example
["Tom", "Dick", "Harry", "Matt", "Annie", "Sam"].indexOf("Annie"); -> 4
* @param {Object} _object
* @return {Number}
*/
Array.prototype.indexOf = function(_object) {

	var me = this;
	var result = -1;
	this.inject(0, function(_memo, _value, _index) {

		if(_object == null) {

			if(_value == null) {

				result = _index;
				me.$break();
			}
		} else if ((_value.equals &&_value.equals(_object)) || _value == _object) {

			result = _index;
			me.$break();
		}

		return _memo;
	});
	return result;
}

/**
* Returns the index in this list of the last occurence of the specified element, or -1 if the list does not contain this element.
* @param {Object} _object
* @return {Number}
*/
Array.prototype.lastIndexOf = function(_object) {

	var result = -1;
	this.inject(0, function(_memo, _value, _index) {

		if(_object == null) {

			if(_value == null) {

				result = _index;
			}
		} else if (_value.equals(_object)) {

			result = _index;
		}

		return _memo;
	});
	return result;
}

/**
* Returns whether the specified object is contained in the array.
* @param {Object} _object Parameter could be null. If there is a null element in the array, true will be returned.
* @return {Boolean}
*/
Array.prototype.contains = function(_object) {

	var me = this;
	if(_object == null) {

		// check for null
		var result = false;
		this.each(function(_value) {

			if(_value == null) {

				result = true;
				me.$break();
			}
		});
		return result;

	} else {

		return this.include(_object);
	}
};

/**
* Returns whether the elements in the specified array exist in the array.
* @param {array}
* @return {Boolean}
*/
Array.prototype.containsAll = function(_array) {

	var me = this;
	var result = true;
	_array.each(function(_value) {

		if(!me.contains(_value)) {

			result = false;
			me.$break();
		}
	});
	return result;
}

/**
* Returns the top item off of this stack without removing it.
* @return {Object}
*/
Array.prototype.peek = function() {

	var size = this.size();

	if(size == 0) {
		return null;
	}

	return this.get(size - 1);
}

/**
* Returns the one-based position of the distance from the top that the specified object exists on this stack, where the top-most element is considered to be at distance 1.
* @param {Object} _object
* @return {Number}
*/
Array.prototype.search = function(_object) {

	var index = this.lastIndexOf(_object);
	if(index > -1) {

		return this.size() - index;
	}
	return -1;
}

/**
* Returns the size of the array.
* @return {Number}
*/
Array.prototype.size = function() {

	return this.length;
};

/**
* Returns true|false if the array is empty
* @return {Boolean}
*/
Array.prototype.isEmpty = function() {

	return this.size() == 0;
};

/**
* Returns a view of the portion of this list between fromIndex, inclusive, and toIndex, exclusive.
* @param {Number} _fromIndex
* @param {Number} _toIndex
* @reutrn {List}
*/
Array.prototype.subList = function(_fromIndex, _toIndex) {

	var sublist = [];

	this.inject(0, function(_memo, _value, _index) {

		if(_index >= _fromIndex && _index <= _toIndex) {

			sublist.add(_value);
		}

		return _memo;
	});

	return sublist;
};

// mutating methods

/**
* Replaces the element at the specified position in this list with the specified element. Returns the element previously at the specified position.
* @param {Number} _index
* @param {Object} _object
* @return {Object}
*/
Array.prototype.set = function(_index, _object) {

	this[_index] = _object;
};

/**
* Inserts the specified element at the specified position in this list.
* @param {Number} _index
* @param {Object} _object
*/
Array.prototype.addAt = function(_index, _object) {

	if(_index < 0){return false;}
	// adding at _index, removing 0 items
	this.splice(_index, 0, _object);
	return true;
};

/**
* Inserts all of the elements in the specified array into this list at the specified position.
* @param {Number} _index
* @param {Array} _array
*/
Array.prototype.addAllAt = function(_index, _array) {

	var me = this;
	var modified = false;
	_array.each(function(_object, _itemIndex) {

		me.addAt((_index + _itemIndex), _object);
		modified = true;
	});
	return modified;
};

/**
* Appends the specified element to the end of this list. Returns whether the element was added to the list.
* @param {Object} _object
* @return {Boolean}
*/
Array.prototype.add = function(_object) {

	if(_object && !_object.equals){ _object.equals = function(_object){ return this == _object; }; }

	return this.addAt(this.size(), _object);
};

/**
* Adds all the elements in the specified collecion to the end of the array. Returns whether the current array was modified.
* @param {Array} _array
* @return {Boolean}
*/
Array.prototype.addAll = function(_array) {

	var me = this;
	var modified = false;
	_array.each(function(_value) {

		if(me.add(_value)) {

			modified = true;
		}
	});
	return modified;
};

/**
* Removes and returns the element at the specified position in this list.
* @param {Number} _index
* @return {Object}
*/
Array.prototype.removeAt = function(_index) {

	if(_index < 0){ return false; }

	var items = this.splice(_index, 1);

	if(items.length == 0){ return false; }

	return true;
};

/**
* Removes the specified object from the array. Return whether the object was removed.
* @param {Object} _object
* @return {Boolean}
*/
Array.prototype.remove = function(_object) {

	var index = this.indexOf(_object);
	if(index > -1) {

		return this.removeAt(index);
	}
	return false;
};

/**
* Removes all the elements in the specified array from the current array. Returns whether the current array was modified.
* @param {Array} _array
* @return {Boolean}
*/
Array.prototype.removeAll = function(_array) {

	var me = this;
	var modified = false;
	_array.each(function(_value) {

		if(me.remove(_value)) {

			modified = true;
		}
	});
	return modified;
};

/**
* Removes all elements in the current array if they are not in the specified array. Returns whether the current array was modified.
* @param {Object} _array
* @return {Boolean}
*/
Array.prototype.retainAll = function(_array) {

	var me = this;
	var modified = false;

	var tmp = [];
	this.each(function(_value) {

		if(!_array.contains(_value)) {

			tmp[tmp.length] = _value;
			modified = true;
		}
	});

	if(modified) {

		for(var i=0;i<tmp.length;i++) {

			this.remove(tmp[i]);
		}
	}

	return modified;
};

/**
* Clears the array
*/
Array.prototype.clear = function() {

	this.length = 0;
};

// sorting methods

Array.prototype.getDefaultComparator = function(_reversal) {

	return function(_objectA, _objectB) {

		return Array.defaultCompare(_objectA, _objectB, _reversal);
	};
}

Array.prototype.getNumberComparator = function(_reversal) {

	return function(_objectA, _objectB) {

		return Array.numberCompare(_objectA, _objectB, _reversal);
	};
}

Array.prototype.getStringComparator = function(_reversal, _insensitiveComparator) {

	return function(_objectA, _objectB) {

		return Array.stringCompare(_objectA, _objectB, _reversal, _insensitiveComparator);
	};
}

Array.prototype.getDateComparator = function(_reversal) {

	return function(_objectA, _objectB) {

		return Array.dateCompare(_objectA, _objectB, _reversal);
	};
}

Array.prototype.getCurrencyComparator = function(_reversal) {

	return function(_objectA, _objectB) {

		return Array.currencyCompare(_objectA, _objectB, _reversal);
	};
}

Array.defaultCompare = function(_objectA, _objectB, _reversal) {

	var a = _objectA;
	var b = _objectB;

	var comp;

	if(a < b){ comp = -1; }
	if(a == b){ comp = 0; }
	if(a > b){ comp = 1; }

	if(_reversal) {

		return -1 * comp;
	}

	return comp;
};

Array.numberCompare = function(_objectA, _objectB, _reversal) {

	var a = _objectA;
	var b = _objectB;

	if(Number.isNaN(a)){ a = 0; }
	if(Number.isNaN(b)){ b = 0; }

	return Array.defaultCompare(a, b, _reversal);
};

Array.stringCompare = function(_objectA, _objectB, _reversal, _insensitiveComparator) {

	var a = _objectA;
	var b = _objectB;

	if(_insensitiveComparator) {

		a = a.toLowerCase();
		b = b.toLowerCase();
	}

	return Array.defaultCompare(a, b, _reversal);
};

Array.dateCompare = function(_objectA, _objectB, _reversal) {

	var _10digit = function(_date) {

		return _date.substr(6,4)+_date.substr(0,2)+_date.substr(3,2);
	};

	var _less10digit = function(_date) {

		var yr = _date.substr(6,2);
		if(Number.parseInt(yr) < 50) { yr = "20"+yr; } else { yr = "19"+yr; }
		return yr+_date.substr(0,2)+_date.substr(3,2);
	};

	var a = _objectA;
	var b = _objectB;

	// format month/day/year
	// y2k notes: two digit years less than 50 are treated as 20XX, greater than 50 are treated as 19XX

	if(a.length == 10) {

		a = _10digit(a);

	} else {

		a = _less10digit(a);
	}

	if(b.length == 10) {

		b = _10digit(b);

	} else {

		b = _less10digit(b);
	}

	return Array.defaultCompare(a, b, _reversal);
};

Array.currencyCompare = function(_objectA, _objectB, _reversal) {

	var a = _objectA;
	var b = _objectB;

	a = a.replace(/[^0-9.]/g, '');
	b = b.replace(/[^0-9.]/g, '');

	a = Number.parseFloat(a);
	b = Number.parseFloat(b);

	return Array.defaultCompare(a, b, _reversal);		
};
/**
* @fileOverview String is one of the core classes to native JavaScript. Here, String mixes in the Enumerable class
*  which gives String a lot more functionality. See Enumerable to see all the enumerable methods on String
* @example
*/

/**
* Returns the number of characters in a string.
* @name length
* @type {Number}
*/
/** @ignore */
String.prototype._$length$_ = "don't use me, used for reflection.";

/**
* Returns the character at a specified position.
* @name charAt
* @param {Number} _index
* @return {String}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$charAt$_ = function(_index){throw "don't call me, used for reflection.";}

/**
* Returns the Unicode of the character at a specified position.
* @name charCodeAt
* @param {Number} _index
* @return {Number}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$charCodeAt$_ = function(_index){throw "don't call me, used for reflection.";}

/**
* Joins two or more strings.
* @name concat
* @param {String} _string. String#concat(stringX,stringX,...,stringX) allowed.
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$concat$_ = function(_string, _n1, _n2){throw "don't call me, used for reflection.";}

/**
* Returns the position of the first occurrence of a specified string value in a string.
* @name indexOf
* @param {String} _string Specifies a string value to search for.
* @param {Number} [_start] Specifies where to start the search.
* @return {Number} This method returns -1 if the string value to search for never occurs.
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$indexOf$_ = function(_string, _start){throw "don't call me, used for reflection.";}

/**
* Returns the position of the last occurrence of a specified string value, searching backwards from the specified position in a string.
* @name lastIndexOf
* @param {String} _string Specifies a string value to search for.
* @param {Number} [_start] Specifies where to start the search.
* @return {Number} This method returns -1 if the string value to search for never occurs.
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$lastIndexOf$_ = function(_string, _start){throw "don't call me, used for reflection.";}

/**
* Searches for a specified value in a string.
* @name match
* @param {String} _regexp Could be /'string'/ or new RegExp("string").
* @return {String}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$match$_ = function(_regexp){throw "don't call me, used for reflection.";}

/**
* Replaces some characters with some other characters in a string.
* @name replace
* @param {String} _regexp Could be /'string'/ or new RegExp("string").
* @param {String} _replacement
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$replace$_ = function(_regexp, _replacement){throw "don't call me, used for reflection.";}

/**
* Searches a string for a specified value. Returns the position of the specified value in the string. If no match was found it returns -1.
* @name search
* @param {String} _regexp Could be /'string'/ or new RegExp("string").
* @return {Number}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$search$_ = function(_regexp){throw "don't call me, used for reflection.";}

/**
* Extracts a part of a string and returns the extracted part in a new string.
* @name slice
* @param {Number} _start Specify where to start the selection.
* @param {Number} _end Specify where to end the selection.
* @return {String}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$slice$_ = function(_start, _end){throw "don't call me, used for reflection.";}

/**
* Splits a string into an array of strings.
* @name split
* @param {String} _delimiter Specifies the character, regular expression, or substring that is used to determine where to split the string.
*  Could be /'string'/ or new RegExp("string").
* @param {Number} [_limit] Specify how many times split should occur. Must be a numeric value.
* @return {Array}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$split$_ = function(_delimiter, _limit){throw "don't call me, used for reflection.";}

/**
* Extracts a specified number of characters in a string, from a start index.
* @name substr
* @param {Number} _start Where to start the extraction.
* @param {Number} [_length] How many characters to extract.
* @return {String}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$substr$_ = function(_start, _length){throw "don't call me, used for reflection.";}

/**
* Extracts the characters in a string between two specified indices.
* @name substring
* @param {Number} _from Where to start the extraction.
* @param {Number} [_to] Where to end the extraction.
* @return {String}
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$substring$_ = function(_from, _to){throw "don't call me, used for reflection.";}

/**
* Displays a string in lowercase letters.
* @name toLowerCase
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$toLowerCase$_ = function(){throw "don't call me, used for reflection.";}

/**
* Displays a string in uppercase letters.
* @name toUpperCase
* @function
* @memberOf String
*/
/** @ignore */
String.prototype._$toUpperCase$_ = function(){throw "don't call me, used for reflection.";}

/**
* Calls encodeURI native on the global object.
*/
String.prototype.encodeURI = function(){return $GLOBAL_OBJECT.encodeURI(this);}

/**
* Calls encodeURIComponent native on the global object.
*/
String.prototype.encodeURIComponent = function(){return $GLOBAL_OBJECT.encodeURIComponent(this);}

/**
* Calls decodeURI native on the global object.
*/
String.prototype.decodeURI = function(){return $GLOBAL_OBJECT.decodeURI(this);}

/**
* Calls decodeURIComponent native on the global object.
*/
String.prototype.decodeURIComponent = function(){return $GLOBAL_OBJECT.decodeURIComponent(this);}

/**
* Calls escape native on the global object.
*/
String.prototype.escape = function(){return $GLOBAL_OBJECT.escape(this);}

/**
* Calls unescape native on the global object.
*/
String.prototype.unescape = function(){return $GLOBAL_OBJECT.unescape(this);}

/**
* Required by the Enumerable mixin. Calls String#iterator(_delimiter, _block) passing a
*  {Object}value and {Number}index to the block.
* @param {String} _delimiter Specifies the character, regular expression, or substring that is used to determine where to split the string.
* @param {Function} _block. Parameters passed to the _block are {Object}array[index] and the {Number}index.
*/
String.prototype.each = function(_delimiter, _block) {

	var me = this;
	var index = 0;
	try {
		this.iterator(_delimiter, function(_value) {
			try {

				_block(_value, index++);

			} catch(e) {

				if (e != Enumerable.$continue){ throw e; }
			}
		});
	} catch(e) {

		if (e != Enumerable.$break){ throw e; }
	}
};

/**
* Iterates through the String array created by String#split, executing the _block and passing a value from String.array[index] as its parameter. Used by String#each(_block).
* @param {Function} _delimiter
* @param {Function} _block
*/
String.prototype.iterator = function(_delimiter, _block) {

	var _parts = this.split(_delimiter);
	for(var i=0;i<_parts.length;i++) {

		_block(_parts[i]);
	}
};

/**
* Returns a new string without spaces on either side.
* @return {String}
*/
String.prototype.trim = function() {

	return this.replace(/^\s+/, "").replace(/\s+$/, "");
};

/**
* Returns a new string escaping HTML reserved characters.
* @return {String}
*/
String.prototype.escapeXml = function() {

	return this.replace(/&/gm, "&amp;")
				.replace(/</gm, "&lt;")
				.replace(/>/gm, "&gt;")
				.replace(/\"/gm, '&quot;')
				.replace(/'/gm, "&apos;");
};

/**
* Returns a new string unescaping HTML reserved characters.
* @return {String}
*/
String.prototype.unescapeXml = function() {

	return this.replace(/&amp;/gm, "&")
				.replace(/&lt;/gm, "<")
				.replace(/&gt;/gm, ">")
				.replace(/&quot;/gm, '"')
				.replace(/&apos;/gm, "'");
};

/**
* Returns whether the string starts with the specified string paramerter.
* @param {String} _string
* @return {Boolean}
*/
String.prototype.startsWith = function(_string) {

	return this.indexOf(_string) == 0;
};

/**
* Returns whether the string ends with the specified string paramerter.
* @param {String} _string
* @return {Boolean}
*/
String.prototype.endsWith = function(_string) {

	return this.lastIndexOf(_string) == (this.length - _string.length);
};

/**
* Returns whether the string has the specified string paramerter within it's contents.
* @param {String} _string
* @return {Boolean}
*/
String.prototype.has = function(_char) {

	return this.indexOf(_char) > -1;
};

/**
* Returns a new string but in reverse order from the original string.
* @return {String}
*/
String.prototype.reverse = function() {

	return this.split("").reverse().join("");
};

/**
* Truncates the string to the specified length. Returns a new string.
* @param {Number} _length The length to truncate the string to.
* @return {String}
*/
String.prototype.truncate = function(_length) {

	return this.slice(0, _length);
};

/**
* Returns a new string capitalizing the original string.
* @return {String}
*/
String.prototype.capitalize = function() {

	return this.charAt(0).toUpperCase() + this.substring(1).toLowerCase();
};

/**
* Returns a new string camel casing the original hyphen or snake case string. Ex. background-image or background_image -> backgroundImage.
* @return {String}
*/
String.prototype.camelCase = function() {

	var _camelized = "";
	var me = this;
	var _once = false;

	if(this.indexOf("-")>-1) {

		this.each("-", function(_value, _index) {

			if(!_value){ me.$continue(); }
			if(!_once) {

				_camelized += _value.charAt(0).toLowerCase() + _value.substring(1);
				_once = true;

			} else {

				_camelized += _value.charAt(0).toUpperCase() + _value.substring(1);
			}
		});

	} else if(this.indexOf("_")>-1) {

		this.each("_", function(_value, _index) {

			if(!_value){ me.$continue(); }
			if(!_once)
			{
				_camelized += _value.charAt(0).toLowerCase() + _value.substring(1);
				_once = true;

			} else {

				_camelized += _value.charAt(0).toUpperCase() + _value.substring(1);
			}
		});

	} else {

		_camelized = this;
	}

	return _camelized;
};

/**
* Returns a new string hyphen casing the original camel case string. Ex. backgroundImage -> background-image.
* @return {String}
*/
String.prototype.hyphenCase = function() {

	var me = this;
	var _hyphenated = "";
	this.each("", function(_value, _index) {

		if(_index == 0 && _value.isUpperCase()) {

			_hyphenated += _value.toLowerCase();

		} else if(_value.isUpperCase()) {

			_hyphenated += "-";
			_hyphenated += _value.toLowerCase();

		} else {

			_hyphenated += _value;
		}
	});
	return _hyphenated;
};

/**
* Returns a new string snake casing the original camel case string. Ex. backgroundImage -> background_image.
* @return {String}
*/
String.prototype.snakeCase = function() {

	var me = this;
	var _snaked = "";
	this.each("", function(_value, _index) {

		if(_index == 0 && _value.isUpperCase()) {

			_snaked += _value.toLowerCase();

		} else if(_value.isUpperCase()) {

			_snaked += "_";
			_snaked += _value.toLowerCase();

		} else {

			_snaked += _value;
		}
	});
	return _snaked;
};

/**
* Returns whether the string has all up cased characters.
* @return {Boolean}
*/
String.prototype.isUpperCase = function() {

	var me = this;
	var _upper = true;
	this.each("", function(_value) {

		if((String.WHITE_SPACE.indexOf(_value) == -1) && (_value.equals(_value.toLowerCase()))) {

			_upper = false;
			me.$break();
		}
	});
	return _upper;
};

/**
* Returns whether the string has all lower cased characters.
* @return {Boolean}
*/
String.prototype.isLowerCase = function() {

	var me = this;
	var _lower = true;
	this.each("", function(_value) {

		if((String.WHITE_SPACE.indexOf(_value) == -1) && (_value.equals(_value.toUpperCase()))) {

			_lower = false;
			me.$break();
		}
	});
	return _lower;
};

/**
* Returns whether the string is blank. Space, tab, new line, carriage return are consider empty.
* @return {Boolean}
*/
String.prototype.isBlank = function() {

	if(this.isEmpty()){ return true; }

	var me = this;
	var _blank = true;
	this.each("", function(_value) {

		if((String.WHITE_SPACE.indexOf(_value) == -1)) {

			_blank = false;
			me.$break();
		}
	});
	return _blank;
};

/**
* Returns whether the string contains all numbers.
* @return {Boolean}
*/
String.prototype.isNumeric = function() {

	if(this.isEmpty()){ return false; }

	var me = this;
	var _digit = true;
	this.each("", function(_value){

		if((String.DIGIT.indexOf(_value) == -1)) {

			_digit = false;
			me.$break();
		}
	});
	return _digit;
};

/**
* Returns whether the string is empty, where this.length == 0.
* @return {Boolean}
*/
String.prototype.isEmpty = function() {

	return this.length == 0;
};

/**
* Returns whether this object is equal to the specified _object.
* @param {Object} _object
* @return {Boolean}
*/
String.prototype.equals = function(_object) {

	if(this === _object){

		return true;
	}

	if(!typeof(_object) == "string") {

		return false;
	}

	return this == _object;
};

String.prototype.$break = Enumerable.$break;
String.prototype.$continue = Enumerable.$continue;
String.prototype.echo = Enumerable.echo;
String.prototype.convertToArray = Enumerable.convertToArray;
String.prototype.all = Enumerable.all;
String.prototype.any = Enumerable.any;
String.prototype.collect = Enumerable.collect;
String.prototype.detect = Enumerable.detect;
String.prototype.findAll = Enumerable.findAll;
String.prototype.grep = Enumerable.grep;
String.prototype.include = Enumerable.include;
String.prototype.inject = Enumerable.inject;
String.prototype.invoke = Enumerable.invoke;
String.prototype.max = Enumerable.max;
String.prototype.min = Enumerable.min;
String.prototype.partition = Enumerable.partition;
String.prototype.pluck = Enumerable.pluck;
String.prototype.reject = Enumerable.reject;
String.prototype.sortBy = Enumerable.sortBy;
String.prototype.zip = Enumerable.zip;
String.prototype.toArray = Enumerable.toArray;
String.prototype.entries = Enumerable.entries;
String.prototype.map = Enumerable.map;
String.prototype.find = Enumerable.find;
String.prototype.member = Enumerable.member;
String.prototype.select = Enumerable.select;

/**
* Returns 0123456789.
*/
String.DIGIT = "0123456789";
/**
* Returns ABCDEFGHIJKLMNOPQRSTUVWXYZ.
*/
String.ALPHA_UPPER_CASE = "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
/**
* Returns abcdefghijklmnopqrstuvwxyz.
*/
String.ALPHA_LOWER_CASE = "abcdefghijklmnopqrstuvwxyz";
/**
* Returns ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.
*/
String.ALPHA = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
/**
* Returns 0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz.
*/
String.ALPHA_NUMERIC = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";
/**
* Returns  [" ", "\n",  "\t", "\r"].
*/
String.WHITE_SPACE = [" ", "\n",  "\t", "\r"];
/**
* Takes the specified Unicode values and returns a string. String.fromCharCode is
*  a class method of the String class.
* @name String.fromCharCode
* @param _numX  String.fromCharCode(numX,numX,...,numX) allowed.
* @return {String}
* @function
*/
/** @ignore */
String._$fromCharCode$_ = function(_n1, _n2, _n3, _n4){throw "don't call me, used for reflection.";};
/**
* Returns the day of the month from a Date object (from 1-31).
* @name getDate
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getDate$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the day of the month in a Date object (from 1-31).
* @name setDate
* @param {Number} _date
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setDate$_ = function(_date){throw "don't call me, used for reflection.";}

/**
* Returns the day of the month from a Date object according to universal time (from 1-31).
* @name getUTCDate
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCDate$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the day of the month in a Date object according to universal time (from 1-31).
* @name setUTCDate
* @param {Number} _date
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setUTCDate$_ = function(_date){throw "don't call me, used for reflection.";}

/**
* Returns the day of the week from a Date object (from 0-6).
* @name getDay
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getDay$_ = function(){throw "don't call me, used for reflection.";}

/**
* Returns the day of the week from a Date object according to universal time (from 0-6).
* @name getUTCDay
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCDay$_ = function(){throw "don't call me, used for reflection.";}

/**
* Returns the month from a Date object (from 0-11).
* @name getMonth
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getMonth$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the month in a Date object (from 0-11).
* @name setMonth
* @param {Number} _month
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setMonth$_ = function(_month){throw "don't call me, used for reflection.";}

/**
* Returns the month from a Date object according to universal time (from 0-11).
* @name getUTCMonth
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCMonth$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the month in a Date object according to universal time (from 0-11).
* @name setUTCMonth
* @param {Number} _month
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setUTCMonth$_ = function(_month){throw "don't call me, used for reflection.";}

/**
* Returns the year, as a four-digit number, from a Date object.
* @name getFullYear
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getFullYear$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the year in a Date object (four digits).
* @name setFullYear
* @param {Number} _fullYear
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setFullYear$_ = function(_fullYear){throw "don't call me, used for reflection.";}

/**
* Returns the four-digit year from a Date object according to universal time.
* @name getUTCFullYear
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCFullYear$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the year in a Date object according to universal time (four digits).
* @name setUTCFullYear
* @param {Number} _fullYear
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setUTCFullYear$_ = function(_fullYear){throw "don't call me, used for reflection.";}

/**
* Returns the year, as a two-digit or a four-digit number, from a Date object. Use getFullYear() instead!
* @name getYear
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getYear$_ = function(){throw "don't call me, used for reflection.";}

/**
* ets the year in the Date object (two or four digits). Use setFullYear() instead!
* @name setYear
* @param {Number} _year
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setYear$_ = function(_year){throw "don't call me, used for reflection.";}

/**
* Returns the hour of a Date object (from 0-23).
* @name getHours
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getHours$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the hour in a Date object (from 0-23).
* @name setHours
* @param {Number} _year
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setHours$_ = function(_hours){throw "don't call me, used for reflection.";}

/**
* Returns the hour of a Date object according to universal time (from 0-23).
* @name getUTCHours
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCHours$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the hour in a Date object according to universal time (from 0-23).
* @name setUTCHours
* @param {Number} _hours
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setUTCHours$_ = function(_hours){throw "don't call me, used for reflection.";}

/**
* Returns the minutes of a Date object (from 0-59).
* @name getMinutes
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getMinutes$_ = function(){throw "don't call me, used for reflection.";}

/**
* Set the minutes in a Date object (from 0-59).
* @name setMinutes
* @param {Number} _minutes
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setMinutes$_ = function(_minutes){throw "don't call me, used for reflection.";}

/**
* Returns the minutes of a Date object according to universal time (from 0-59).
* @name getUTCMinutes
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCMinutes$_ = function(){throw "don't call me, used for reflection.";}

/**
* Set the minutes in a Date object according to universal time (from 0-59).
* @name setUTCMinutes
* @param {Number} _minutes
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setUTCMinutes$_ = function(_minutes){throw "don't call me, used for reflection.";}

/**
* Returns the seconds of a Date object (from 0-59).
* @name getSeconds
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getSeconds$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the seconds in a Date object (from 0-59).
* @name setSeconds
* @param {Number} _seconds
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setSeconds$_ = function(_seconds){throw "don't call me, used for reflection.";}

/**
* Returns the seconds of a Date object according to universal time (from 0-59).
* @name getUTCSeconds
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCSeconds$_ = function(){throw "don't call me, used for reflection.";}

/**
* Set the seconds in a Date object according to universal time (from 0-59).
* @name setUTCSeconds
* @param {Number} _seconds
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setUTCSeconds$_ = function(_seconds){throw "don't call me, used for reflection.";}

/**
* Returns the milliseconds of a Date object (from 0-999).
* @name getMilliseconds
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getMilliseconds$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the milliseconds in a Date object (from 0-999).
* @name setMilliseconds
* @param {Number} _milliseconds
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setMilliseconds$_ = function(_milliseconds){throw "don't call me, used for reflection.";}

/**
* Returns the milliseconds of a Date object according to universal time (from 0-999).
* @name getUTCMilliseconds
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getUTCMilliseconds$_ = function(){throw "don't call me, used for reflection.";}

/**
* Sets the milliseconds in a Date object according to universal time (from 0-999).
* @name setUTCMilliseconds
* @param {Number} _milliseconds
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setUTCMilliseconds$_ = function(_milliseconds){throw "don't call me, used for reflection.";}

/**
* Returns the number of milliseconds since midnight Jan 1, 1970.
* @name getTime
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getTime$_ = function(){throw "don't call me, used for reflection.";}

/**
* Calculates a date and time by adding or subtracting a specified number of milliseconds to/from midnight January 1, 1970.
* @name setTime
* @param {Number} _time
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$setTime$_ = function(_time){throw "don't call me, used for reflection.";}

/**
* Returns the difference in minutes between local time and Greenwich Mean Time (GMT).
* @name getTimezoneOffset
* @return {Number}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$getTimezoneOffset$_ = function(){throw "don't call me, used for reflection.";}

/**
* Returns a string representation of the date. Ex... Fri May 09 2008.
* @name toDateString
* @return {String}
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$toDateString$_ = function(){throw "don't call me, used for reflection.";}

/**
* Converts a Date object, according to Greenwich time, to a string. Use toUTCString() instead!
* @name toGMTString
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$toGMTString$_ = function(){throw "don't call me, used for reflection.";}

/**
* Returns a string representation of the time. Ex... 21:28:42 GMT-0400 (EDT).
* @name toTimeString
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$toTimeString$_ = function(){throw "don't call me, used for reflection.";}

/**
* Converts a Date object, according to universal time, to a string.
* @name toUTCString
* @memberof Date
* @function
*/
/** @ignore */
Date.prototype._$toUTCString$_ = function(){throw "don't call me, used for reflection.";}

/**
* Returns the difference in milliseconds from this date object to _date parameter.
* @return {Number}
* @param {Date} _date
* @return {Number}
*/
Date.prototype.getDifference = function(_date) {

	return this.getTime() - _date.getTime();
}

/**
* Returns whether this date object is before _date parameter.
* @param {Date} _date
* @return {Boolean}
*/
Date.prototype.isBefore = function(_date) {

	return this.getTime() < _date.getTime();
}

/**
* Returns whether this date object is after _date parameter.
* @param {Date} _date
* @return {Boolean}
*/
Date.prototype.isAfter = function(_date) {

	return this.getTime() > _date.getTime();
}

/**
* Returns the number of week in the year. Ex... 29.
* @param {Number} _minimalDaysInFirstWeek
* @return {Number}
*/
// This method was adopted from log4javascript -> http://www.timdown.co.uk/log4javascript
Date.prototype.getWeekInYear = function(_minimalDaysInFirstWeek) {
 
	if(undefined === _minimalDaysInFirstWeek) {

		_minimalDaysInFirstWeek = Date.DEFAULT_MINIMAL_DAYS_IN_FIRST_WEEK;
	}

	var _previousSunday = new Date(this.getTime() - (this.getDay() * Date.ONE_DAY_MILLISECONDS));
	_previousSunday = new Date(_previousSunday.getFullYear(), _previousSunday.getMonth(), _previousSunday.getDate());

	var _startOfYear = new Date(this.getFullYear(), 0, 1);

	var _numberOfSundays = _previousSunday.isBefore(_startOfYear) ?
							0 : 1 + Math.floor((_previousSunday.getTime() - _startOfYear.getTime()) / Date.ONE_WEEK_MILLISECONDS);

	var _weekInYear = _numberOfSundays;

	var _numberOfDaysInFirstWeek = 7 - _startOfYear.getDay();
	if(_numberOfDaysInFirstWeek >= _minimalDaysInFirstWeek) {

		_weekInYear++;
	}

	return _weekInYear;
}

/**
* Returns the number of week in the month. Ex... 4.
* @param {Number} _minimalDaysInFirstWeek
* @return {Number}
*/
// This method was adopted from log4javascript -> http://www.timdown.co.uk/log4javascript
Date.prototype.getWeekInMonth = function(_minimalDaysInFirstWeek) {

	if(undefined === _minimalDaysInFirstWeek) {

		_minimalDaysInFirstWeek = Date.DEFAULT_MINIMAL_DAYS_IN_FIRST_WEEK;
	}

	var _previousSunday = new Date(this.getTime() - (this.getDay() * Date.ONE_DAY_MILLISECONDS));
	_previousSunday = new Date(_previousSunday.getFullYear(), _previousSunday.getMonth(), _previousSunday.getDate());

	var _startOfMonth = new Date(this.getFullYear(), this.getMonth(), 1);

	var _numberOfSundays = _previousSunday.isBefore(_startOfMonth) ?
							0 : Math.floor((_previousSunday.getTime() - _startOfMonth.getTime()) / Date.ONE_WEEK_MILLISECONDS);

	var _weekInMonth = _numberOfSundays;

	var _numberOfDaysInFirstWeek = 7 - _startOfMonth.getDay();
	if(_numberOfDaysInFirstWeek >= _minimalDaysInFirstWeek) {

		_weekInMonth++;
	}

	return _weekInMonth;
}

/**
* Returns the day in the year. Ex... 360.
* @return {Number}
*/
// This method was adopted from log4javascript -> http://www.timdown.co.uk/log4javascript
Date.prototype.getDayInYear = function() {

	var _startOfYear = new Date(this.getFullYear(), 0, 1);
	return 1 + Math.floor((this.getTime() - _startOfYear.getTime()) / Date.ONE_DAY_MILLISECONDS);
}

/**
* Not Implemented.
* @return {String}
*/
Date.prototype.getTimeZone = function() {

	var _timezone = "xxx";

	//var _timezoneOffset = this.getTimezoneOffset();

	return _timezone;
}

/**
* Returns whether this object is equal to the specified _object.
* @param {Object} _object
* @return {Boolean}
*/
Date.prototype.equals = function(_object) {

	if(this === _object) {

		return true;
	}
	if(this.constructor != _object.constructor) {

		return false;
	}

	return this.getTime() == _object.getTime();
}

/**
* The long version of the months, ex. January.
*/
Date.MONTH_NAMES = ["January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"];
/**
* The short verions of the months, ex. Jan.
*/
Date.MONTH_NAMES_SHORT = ["Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"];

/**
* The long verion of the days of the week, ex. Sunday.
*/
Date.DAY_NAMES = ["Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"];
/**
* The short version of the days of the week, ex. Sun.
*/
Date.DAY_NAMES_SHORT = ["Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"];

/**
* The number of milliseconds in a day.
*/
Date.ONE_DAY_MILLISECONDS = 24 * 60 * 60 * 1000;
/**
* The number of milliseconds in a week.
*/
Date.ONE_WEEK_MILLISECONDS = 7 * Date.ONE_DAY_MILLISECONDS;

/** @ignore */
Date.DEFAULT_MINIMAL_DAYS_IN_FIRST_WEEK = 1;

/**
* Takes a date string and returns the number of milliseconds since midnight of January 1, 1970.
* @name Date.parse
* @param {String} _dateString A string representing a date.
* @function
*/
/** @ignore */
Date._$parse$_ = function(_dateString){throw "don't call me, used for reflection.";};
/**
* Takes a date and returns the number of milliseconds since midnight of January 1, 1970 according to universal time.
* @name Date.UTC
* @param {Number} _year
* @param {Number} _month
* @param {Number} _day
* @param {Number} [_hours]
* @param {Number} [_minutes]
* @param {Number} [_seconds]
* @param {Number} [_ms]
* @function
*/
/** @ignore */
Date._$UTC$_ = function(_year, _month, _day, _hours, _minutes, _seconds, _ms){throw "don't call me, used for reflection.";};
/**
* Returns Euler's constant (approx. 2.718).
* @name E
*/
/** @ignore */
Math._$E$_ = "don't use me, used for reflection.";
/**
* Returns the natural logarithm of 10 (approx. 2.302).
* @name LN10
*/
/** @ignore */
Math._$LN10$_ = "don't use me, used for reflection.";
/**
* Returns the natural logarithm of 2 (approx. 0.693).
* @name LN2
*/
/** @ignore */
Math._$LN2$_ = "don't use me, used for reflection.";
/**
* Returns the base-10 logarithm of E (approx. 0.434).
* @name LOG10E
*/
/** @ignore */
Math._$LOG10E$_ = "don't use me, used for reflection.";
/**
* Returns the base-2 logarithm of E (approx. 1.414).
* @name LOG2E
*/
/** @ignore */
Math._$LOG2E$_ = "don't use me, used for reflection.";
/**
* Returns PI (approx. 3.14159).
* @name PI
*/
/** @ignore */
Math._$PI$_ = "don't use me, used for reflection.";
/**
* Returns the square root of 1/2 (approx. 0.707).
* @name SQRT1_2
*/
/** @ignore */
Math._$SQRT1_2$_ = "don't use me, used for reflection.";
/**
* Returns the square root of 2 (approx. 1.414).
* @name SQRT2
*/
/** @ignore */
Math._$SQRT2$_ = "don't use me, used for reflection.";

/**
* Returns the absolute value of a number.
* @name Math.abs
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$abs$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the arccosine of a number.
* @name Math.acos
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$acos$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the arcsine of a number.
* @name Math.asin
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$asin$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the arctangent of x as a numeric value between -PI/2 and PI/2 radians.
* @name Math.atan
* @param {Number}
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$atan$_ = function(_nY, _nX){throw "don't call me, used for reflection.";};
/**
* Returns the angle theta of an (x,y) point as a numeric value between -PI and PI radians.
* @name Math.atan2
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$atan2$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the value of a number rounded upwards to the nearest integer.
* @name Math.ceil
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$ceil$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the cosine of a number.
* @name Math.cos
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$cos$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the value of E^x.
* @name Math.exp
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$exp$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the value of a number rounded downwards to the nearest integer.
* @name Math.floor
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$floor$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the natural logarithm (base E) of a number.
* @name Math.log
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$log$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the number with the highest value of x and y.
* @name Math.max
* @param {Number} _args Math.max(_num1, _num2, ...._numX) allowed.
* @return {Number}
* @function
*/
/** @ignore */
Math._$max$_ = function(_args){throw "don't call me, used for reflection.";};
/**
* Returns the number with the lowest value of x and y.
* @name Math.min
* @param {Number} _args Math.min(_num1, _num2, ...._numX) allowed.
* @return {Number}
* @function
*/
/** @ignore */
Math._$min$_ = function(_args){throw "don't call me, used for reflection.";};
/**
* Returns the value of x to the power of y.
* @name Math.pow
* @param {Number}
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$pow$_ = function(_x, _y){throw "don't call me, used for reflection.";};
/**
* Returns a random number between 0 and 1.
* @name Math.randon
* @return {Number}
* @function
*/
/** @ignore */
Math._$randon$_ = function(){throw "don't call me, used for reflection.";};
/**
* Rounds a number to the nearest integer.
* @name Math.round
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$round$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the sine of a number.
* @name Math.sin
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$sin$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the square root of a number.
* @name Math.sqrt
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$sqrt$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Returns the tangent of an angle.
* @name Math.tan
* @param {Number}
* @return {Number}
* @function
*/
/** @ignore */
Math._$tan$_ = function(_n){throw "don't call me, used for reflection.";};
/**
* Specifies if the "g" modifier is set.
* @name global
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$global$_ = "don't use me, used for reflection.";
/**
* Specifies if the "i" modifier is set.
* @name ignoreCase
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$ignoreCase$_ = "don't use me, used for reflection.";
/**
* Specifies if the "m" modifier is set.
* @name multiline
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$multiline$_ = "don't use me, used for reflection.";
/**
* The string on which the pattern match is performed.
* @name input
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$input$_ = "don't use me, used for reflection.";
/**
* An integer specifying the index at which to start the next match.
* @name lastIndex
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$lastIndex$_ = "don't use me, used for reflection.";
/**
* The last matched characters.
* @name lastMatch
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$lastMatch$_ = "don't use me, used for reflection.";
/**
* The last matched parenthesized substring.
* @name lastParen
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$lastParen$_ = "don't use me, used for reflection.";
/**
* The substring in front of the characters most recently matched.
* @name leftContext
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$leftContext$_ = "don't use me, used for reflection.";
/**
* The substring after the characters most recently matched.
* @name rightContext
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$rightContext$_ = "don't use me, used for reflection.";
/**
* The text used for pattern matching.
* @name source
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$source$_ = "don't use me, used for reflection.";

/**
* Change the regular expression (what to search for).
* @name compile
* @param {String} _string
* @function
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$compile$_ = function(_string){throw "don't call me, used for reflection.";}

/**
* Search a string for a specified value. Returns the found value and remembers the position.
* @name exec
* @param {String} _string
* @return {String}
* @function
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$exec$_ = function(_string){throw "don't call me, used for reflection.";}

/**
* Search a string for a specified value. Returns true or false.
* @name test
* @param {String} _string
* @return {Boolean}
* @function
* @memberOf RegExp
*/
/** @ignore */
RegExp.prototype._$test$_ = function(_string){throw "don't call me, used for reflection.";}

/**
* Returns whether this object is equal to the specified _object.
* @param {Object} _object
* @return {Boolean}
*/
RegExp.prototype.equals =  function(_object) {

	if (this === _object) {

		return true;
	}
	if(this.constructor != _object.constructor) {

		return false;
	}

	return this == _object;
};
var Hash = function(){this.init.apply(this, arguments);};
exports.Hash = Hash;

Hash.prototype.init = function() {

	this.data = {};
	this.dataSize = 0;
};

/**
* Returns whether this hash is equal to the specified hash.
* @param {Object} _object
* @return {Boolean}
*/
Hash.prototype.equals = function(_object) {

	if (this === _object) {

		return true;
	}
	if(this.constructor != _object.constructor) {

		return false;
	}

	// compare lengths
	if(this.size() != _object.size()){ return false; }

	var me = this;
	var result = true;
	this.each(function(_value, _key) {

		if(_value == null) {

			if(!(_object.get(_key) == null && _object.containsKey(_key))) {

				result = false;
				me.$break();
			}

		} else {

			if(!_value.equals(_object.get(_key))) {

				result = false;
				me.$break();
			}
		}
	});

	return result;
};

/**
* Returns a String representation of a Hash.
* @return {Stirng}
*/
Hash.prototype.toString = function() {

	var me = this;
	var sb = "";
	sb += "{";

	var index = 0;
	this.each(function(_value, _key)  {

		if(_key != null) {

			sb += _key.toString();

		} else {

			sb += "null";
		}

		sb += " = ";

		if(_value != null) {

			sb += _value.toString();

		} else {

			sb += "null";
		}

		if(index != (me.size() -1)){sb += ", ";}
		index++;
	});

	sb += "}";
	return sb;
};

// enumerable methods
/**
* Required by the Enumerable mixin. Calls AbstractMap#iterator(_block) passing a {Object}value and {Object}key to the block.
* @param {Function} _block. Parameters passed to the _block are <value>Object, <key>Object.
*/
Hash.prototype.each = function(_block) {

	var me = this;
	try {

		this.iterator(function(_entry) {

			try {

				_block(_entry.value, _entry.key);

			} catch(e) {

				if (e != Enumerable.$continue){ throw e; }
			}
		});

	} catch(e) {

		if (e != Enumerable.$break){ throw e; }
	}
};

/**
* Iterate over the hash of entries.
* @param {Function} _block The function to execute for every key-value mapping in the hash.
*/
Hash.prototype.iterator = function(_block) {

	for(var prop in this.data) {

		var entry = {};
		entry.key = prop;
		entry.value = this.data[prop];

		_block(entry);
	}
};

Hash.prototype.$break = Enumerable.$break;
Hash.prototype.$continue = Enumerable.$continue;
Hash.prototype.echo = Enumerable.echo;
Hash.prototype.convertToArray = Enumerable.convertToArray;
Hash.prototype.all = Enumerable.all;
Hash.prototype.any = Enumerable.any;
Hash.prototype.collect = Enumerable.collect;
Hash.prototype.detect = Enumerable.detect;
Hash.prototype.findAll = Enumerable.findAll;
Hash.prototype.grep = Enumerable.grep;
Hash.prototype.include = Enumerable.include;
Hash.prototype.inject = Enumerable.inject;
Hash.prototype.invoke = Enumerable.invoke;
Hash.prototype.max = Enumerable.max;
Hash.prototype.min = Enumerable.min;
Hash.prototype.partition = Enumerable.partition;
Hash.prototype.pluck = Enumerable.pluck;
Hash.prototype.reject = Enumerable.reject;
Hash.prototype.sortBy = Enumerable.sortBy;
Hash.prototype.zip = Enumerable.zip;
Hash.prototype.toArray = Enumerable.toArray;
Hash.prototype.entries = Enumerable.entries;
Hash.prototype.map = Enumerable.map;
Hash.prototype.find = Enumerable.find;
Hash.prototype.member = Enumerable.member;
Hash.prototype.select = Enumerable.select;

// querying methods
/**
* Returns the value to which this hash maps the specified key.
* @param {Object} _key
* @return {Object}
*/
Hash.prototype.get = function(_key) {

	var me = this;
	var result = null;
	this.each(function(_value, _hashKey) {

		if(_key == null) {

			if(_hashKey == "null") {

				result = _value;
				me.$break();
			}

		} else {

			if(_hashKey != "null" && _hashKey.equals(_key)) {

				result = _value;
				me.$break();
			}
		}
	});
	return result;
};

/**
* Returns a array view of the keys contained in this map.
* @return {Array}
*/
Hash.prototype.keys = function() {

	var array = [];
	this.each(function(_value, _key) {

		array.add(_key);
	});
	return array;
};

/**
* Returns a array view of the values contained in this map.
* @return {Array}
*/
Hash.prototype.values = function() {

	var array = [];
	this.each(function(_value, _key) {

		array.add(_value);
	});
	return array;
};

/**
* Returns true if this hash contains a mapping for the specified key.
* @param {Object} _key
* @return {Boolean]
*/
Hash.prototype.containsKey = function(_key) {

	return this.keys().contains(_key);
};

/**
* Returns true if this hash maps one or more keys to this value.
* @param {Object} _value
* @return {Boolean]
*/
Hash.prototype.containsValue = function(_value) {

	return this.values().contains(_value);
};

/**
* Returns true if this hash contains no key-value mappings.
* @return {Boolean}
*/
Hash.prototype.isEmpty = function() {

	return this.size() == 0;
};

/**
* Returns the number of key-value mappings in this hash.
* @return {Number}
*/
Hash.prototype.size = function() {

	return this.dataSize;
};

// mutating methods
/**
* Associates the specified value with the specified key in this hash.
* @param {Object} _key
* @param {Object} _value
*/
Hash.prototype.put = function(_key, _value) {

	if(_key && !_key.equals) { _key.equals = function(_object){ return this == _object; }; }
	if(_value && !_value.equals){ _value.equals = function(_object){ return this == _object; }; }

	var result = null;
	if(this.containsKey(_key)) {

		result = this.remove(_key);
	}

	this.data[_key] = _value;

	this.dataSize++;

	return result;
};

/**
* Removes and returns the mapping for this key from this hash if it is present.
* @param {Object} _key
* @return {Object}
*/
Hash.prototype.remove = function(_key) {

	var _index = this.keys().indexOf(_key);
	if(_index <= -1) {

		return null;
	}

	var _value = this.get(_key);

	delete this.data[_key];

	this.dataSize--;

	return _value;
};

/**
* Copies all of the mappings from the specified Hash to this Hash.
* @param {Map} _hash
*/
Hash.prototype.putAll = function(_hash) {

	var me = this;
	_hash.each(function(_value, _key) {

		me.put(_key, _value);
	});
};

/**
* Removes all mappings from this map.
*/
Hash.prototype.clear = function() {

	var me = this;
	this.keys().each(function(_value, _key) {

		me.remove(_value);
	});
};
var HashSet = function(){this.init.apply(this, arguments);};
exports.HashSet = HashSet;

HashSet.prototype = new Array();
HashSet.prototype.constructor = HashSet;
HashSet.parentclass = Array.prototype;

HashSet.prototype.init = function() {

	this.entryItems = new Hash();

	var _present = {};

	/** @ignore */
	this.__dummy__ = function() {

		return _present;
	};
};

/**
* Returns whether this HashSet is equal to the specified HashSet.
* @param {Object} _object
* @return {Boolean}
*/
HashSet.prototype.equals = function(_object) {

	if (this === _object) {

		return true;
	}
	if(this.constructor != _object.constructor) {

		return false;
	}

	// compare lengths
	if(this.size() != _object.size()){ return false; }

	return this.containsAll(_object);
};

// enumerable methods
/**
* Iterate over the HashSet of elements.
* @param {Function} _block The function to execute for every element in the HashSet.
*/
HashSet.prototype.iterator = function(_block) {

	var keys = this.entryList().keys();
	var size = keys.size();
	for(var i=0;i<size;i++) {

		_block(keys.get(i));
	}
};

// querying methods
/**
* Returns true if this HashSet contains the specified element.
* @return {Boolean}
*/
HashSet.prototype.contains = function(_object) {

	return this.entryItems.containsKey(_object);
};

/**
* Returns true if this HashSet contains no elements.
* @return {Boolean}
*/
HashSet.prototype.isEmpty = function() {

	return this.entryItems.isEmpty();
};

/**
* Returns the number of elements in this HashSet.
* @return {Number}
*/
HashSet.prototype.size = function() {

	return this.entryItems.size();
};

// mutating methods
/**
* Adds the specified element to this HashSet if it is not already present.
* @param {Object} _object
* @return {Boolean}
*/
HashSet.prototype.add = function(_object) {

	return (this.entryItems.put(_object, this.__dummy__()) == null);
};

/**
* Removes the specified element from this HashSet if it is present.
* @param {Object} _object
* @return {Bolean}
*/
HashSet.prototype.remove = function(_object) {

	return (this.entryItems.remove(_object) == this.__dummy__());
};

/**
* Removes all of the elements from this HashSet.
*/
HashSet.prototype.clear = function() {

	this.entryItems.clear();
};
/**
* @class Closure
* @constructor
* @description A Closure is a function that is evaluated in an environment containing one or more bound variables.
*  When called, the function can access these variables. In this case the _block parameter in the bind method will have
*  access to variable in the same scope as where the closure is defined.
*/

/**
* Returns a Function. The _object parameter is appended to the _args parameter when the _block is excuted. If the
*  Closure is bound to an event, the event is appended to the _args parameter when the _block is executed.
* @param {Function} _block Block to execute
* @param {Object} _object Context Object
* @param {Array} [_args] Array of parameters to pass into the block
* @return {Function}
*/
var Closure = function(){this.init.apply(this, arguments);};
exports.Closure = Closure;

Closure.prototype.init = function() {};
Closure.prototype.bind = function(_block, _object, _args_) {

	var _args = _args_ || [];

	return function(_event_) {

		_args = _args.concat([_object]);

		// both _event or global object event could be null, which means the result could be null if this closure
		// is not bound to an event
		var _event = _event_ || $GLOBAL_OBJECT.event;
		if(_event) {

			_args = _args.concat([_event]);
		}

		return _block.apply(_object, _args);
	};
};
/**
* @class AsyncTask
* @constructor
* @description This is an API wrapping the global objects setTimeout / setInterval methods.
* @param {Function} _block Block to execute.
* @param {Object} _object Context object.
* @param {Array} _args Array of parameters to pass into the block.
* @param {Number} _milliseconds Specify in milliseconds when to execute. If the AsyncTask is a daemon AsyncTask, the milliseconds will indicate
*  the intervals the block will execute. If the AsyncTask is not a daemon AsyncTask, the milliseconds will indicate the delay until the
*  block will execute.
* @param {String} _name Name of the AsyncTask.
*/
var AsyncTask = function(){this.init.apply(this, arguments);};
exports.AsyncTask = AsyncTask;

AsyncTask.prototype.init = function(_block, _object, _args, _milliseconds, _name) {

	this.block = _block;
	this.object = _object;
	this.args = _args;
	this.milliseconds = _milliseconds;
	this.name = _name;

	this.id = 0;
	this.daemon = false;
};

/**
* Starts the jsx.ui.util.AsyncTask.
*/
AsyncTask.prototype.start = function() {

	if(!this.milliseconds || this.milliseconds < 0){ this.milliseconds = 1; }
	if(this.daemon) {

		// execute blocktion every milliseconds
		this.id = setInterval(new Closure().bind(this.block, this.object, this.args || []), this.milliseconds);

	} else {

		// execute blocktion in milliseconds
		this.id = setTimeout(new  Closure().bind(this.block, this.object, this.args || []), this.milliseconds);
	}
};

/**
* Stops the jsx.ui.util.AsyncTask.
*/
AsyncTask.prototype.stop = function() {

	if(this.daemon) {

		clearInterval(this.id);

	} else {

		clearTimeout(this.id);
	}
};

/**
* Makes the AsyncTask sleep for a number of milliseconds.
* @param {Number} _milliseconds How long to sleep for.
*/
AsyncTask.prototype.sleep = function(_milliseconds) {

	this.stop();

	new AsyncTask(function() {

		this.start();
	}, this.object, [], _milliseconds).start();
};

/**
* Returns the AsyncTask ID.
* @return {Number}
*/
AsyncTask.prototype.getID = function() {

	return this.id;
};

/**
* Returns the AsyncTask Name.
* @return {String}
*/
AsyncTask.prototype.getName = function() {

	return this.name;
};

/**
* Set the AsyncTask Name.
* @param {String} _name
* @return {AsyncTask}
*/
AsyncTask.prototype.setName = function(_name) {

	this.name = _name;
	return this;
};

/**
* Returns whether the AsyncTask is a Daemon jsx.ui.util.AsyncTask.
* @return {Boolean}
*/
AsyncTask.prototype.isDaemon = function() {

	return this.daemon;
};

/**
* Set whether the AsyncTask should be a Daemon jsx.ui.util.AsyncTask.
* @param {Boolean} _isDaemon
* @return {AsyncTask}
*/
AsyncTask.prototype.setDaemon = function(_isDaemon) {

	this.daemon = _isDaemon;
	return this;
};

/**
* Returns the String representation of the jsx.ui.util.AsyncTask.
* @return {String}
*/
AsyncTask.prototype.toString = function() {

	var _tmp = "AsyncTask -> ID: " + this.getID();
	if(this.name){ _tmp += " Name: " + this.name; }
	return _tmp;
};
