/**
* @class AsyncTask
* @constructor
* @description This is an API wrapping the global objects setTimeout / setInterval methods.
* @param {Function} _block Block to execute.
* @param {Object} _object Context object.
* @param {Array} _args Array of parameters to pass into the block.
* @param {Number} _milliseconds Specify in milliseconds when to execute. If the AsyncTask is a daemon AsyncTask, the milliseconds will indicate
*  the intervals the block will execute. If the AsyncTask is not a daemon AsyncTask, the milliseconds will indicate the delay until the
*  block will execute.
* @param {String} _name Name of the AsyncTask.
*/
var AsyncTask = function(){this.init.apply(this, arguments);};
exports.AsyncTask = AsyncTask;

AsyncTask.prototype.init = function(_block, _object, _args, _milliseconds, _name) {

	this.block = _block;
	this.object = _object;
	this.args = _args;
	this.milliseconds = _milliseconds;
	this.name = _name;

	this.id = 0;
	this.daemon = false;
};

/**
* Starts the jsx.ui.util.AsyncTask.
*/
AsyncTask.prototype.start = function() {

	if(!this.milliseconds || this.milliseconds < 0){ this.milliseconds = 1; }
	if(this.daemon) {

		// execute blocktion every milliseconds
		this.id = setInterval(new Closure().bind(this.block, this.object, this.args || []), this.milliseconds);

	} else {

		// execute blocktion in milliseconds
		this.id = setTimeout(new  Closure().bind(this.block, this.object, this.args || []), this.milliseconds);
	}
};

/**
* Stops the jsx.ui.util.AsyncTask.
*/
AsyncTask.prototype.stop = function() {

	if(this.daemon) {

		clearInterval(this.id);

	} else {

		clearTimeout(this.id);
	}
};

/**
* Makes the AsyncTask sleep for a number of milliseconds.
* @param {Number} _milliseconds How long to sleep for.
*/
AsyncTask.prototype.sleep = function(_milliseconds) {

	this.stop();

	new AsyncTask(function() {

		this.start();
	}, this.object, [], _milliseconds).start();
};

/**
* Returns the AsyncTask ID.
* @return {Number}
*/
AsyncTask.prototype.getID = function() {

	return this.id;
};

/**
* Returns the AsyncTask Name.
* @return {String}
*/
AsyncTask.prototype.getName = function() {

	return this.name;
};

/**
* Set the AsyncTask Name.
* @param {String} _name
* @return {AsyncTask}
*/
AsyncTask.prototype.setName = function(_name) {

	this.name = _name;
	return this;
};

/**
* Returns whether the AsyncTask is a Daemon jsx.ui.util.AsyncTask.
* @return {Boolean}
*/
AsyncTask.prototype.isDaemon = function() {

	return this.daemon;
};

/**
* Set whether the AsyncTask should be a Daemon jsx.ui.util.AsyncTask.
* @param {Boolean} _isDaemon
* @return {AsyncTask}
*/
AsyncTask.prototype.setDaemon = function(_isDaemon) {

	this.daemon = _isDaemon;
	return this;
};

/**
* Returns the String representation of the jsx.ui.util.AsyncTask.
* @return {String}
*/
AsyncTask.prototype.toString = function() {

	var _tmp = "AsyncTask -> ID: " + this.getID();
	if(this.name){ _tmp += " Name: " + this.name; }
	return _tmp;
};
