var Hash = function(){this.init.apply(this, arguments);};
exports.Hash = Hash;

Hash.prototype.init = function() {

	this.data = {};
	this.dataSize = 0;
};

/**
* Returns whether this hash is equal to the specified hash.
* @param {Object} _object
* @return {Boolean}
*/
Hash.prototype.equals = function(_object) {

	if (this === _object) {

		return true;
	}
	if(this.constructor != _object.constructor) {

		return false;
	}

	// compare lengths
	if(this.size() != _object.size()){ return false; }

	var me = this;
	var result = true;
	this.each(function(_value, _key) {

		if(_value == null) {

			if(!(_object.get(_key) == null && _object.containsKey(_key))) {

				result = false;
				me.$break();
			}

		} else {

			if(!_value.equals(_object.get(_key))) {

				result = false;
				me.$break();
			}
		}
	});

	return result;
};

/**
* Returns a String representation of a Hash.
* @return {Stirng}
*/
Hash.prototype.toString = function() {

	var me = this;
	var sb = "";
	sb += "{";

	var index = 0;
	this.each(function(_value, _key)  {

		if(_key != null) {

			sb += _key.toString();

		} else {

			sb += "null";
		}

		sb += " = ";

		if(_value != null) {

			sb += _value.toString();

		} else {

			sb += "null";
		}

		if(index != (me.size() -1)){sb += ", ";}
		index++;
	});

	sb += "}";
	return sb;
};

// enumerable methods
/**
* Required by the Enumerable mixin. Calls AbstractMap#iterator(_block) passing a {Object}value and {Object}key to the block.
* @param {Function} _block. Parameters passed to the _block are <value>Object, <key>Object.
*/
Hash.prototype.each = function(_block) {

	var me = this;
	try {

		this.iterator(function(_entry) {

			try {

				_block(_entry.value, _entry.key);

			} catch(e) {

				if (e != Enumerable.$continue){ throw e; }
			}
		});

	} catch(e) {

		if (e != Enumerable.$break){ throw e; }
	}
};

/**
* Iterate over the hash of entries.
* @param {Function} _block The function to execute for every key-value mapping in the hash.
*/
Hash.prototype.iterator = function(_block) {

	for(var prop in this.data) {

		var entry = {};
		entry.key = prop;
		entry.value = this.data[prop];

		_block(entry);
	}
};

Hash.prototype.$break = Enumerable.$break;
Hash.prototype.$continue = Enumerable.$continue;
Hash.prototype.echo = Enumerable.echo;
Hash.prototype.convertToArray = Enumerable.convertToArray;
Hash.prototype.all = Enumerable.all;
Hash.prototype.any = Enumerable.any;
Hash.prototype.collect = Enumerable.collect;
Hash.prototype.detect = Enumerable.detect;
Hash.prototype.findAll = Enumerable.findAll;
Hash.prototype.grep = Enumerable.grep;
Hash.prototype.include = Enumerable.include;
Hash.prototype.inject = Enumerable.inject;
Hash.prototype.invoke = Enumerable.invoke;
Hash.prototype.max = Enumerable.max;
Hash.prototype.min = Enumerable.min;
Hash.prototype.partition = Enumerable.partition;
Hash.prototype.pluck = Enumerable.pluck;
Hash.prototype.reject = Enumerable.reject;
Hash.prototype.sortBy = Enumerable.sortBy;
Hash.prototype.zip = Enumerable.zip;
Hash.prototype.toArray = Enumerable.toArray;
Hash.prototype.entries = Enumerable.entries;
Hash.prototype.map = Enumerable.map;
Hash.prototype.find = Enumerable.find;
Hash.prototype.member = Enumerable.member;
Hash.prototype.select = Enumerable.select;

// querying methods
/**
* Returns the value to which this hash maps the specified key.
* @param {Object} _key
* @return {Object}
*/
Hash.prototype.get = function(_key) {

	var me = this;
	var result = null;
	this.each(function(_value, _hashKey) {

		if(_key == null) {

			if(_hashKey == "null") {

				result = _value;
				me.$break();
			}

		} else {

			if(_hashKey != "null" && _hashKey.equals(_key)) {

				result = _value;
				me.$break();
			}
		}
	});
	return result;
};

/**
* Returns a array view of the keys contained in this map.
* @return {Array}
*/
Hash.prototype.keys = function() {

	var array = [];
	this.each(function(_value, _key) {

		array.add(_key);
	});
	return array;
};

/**
* Returns a array view of the values contained in this map.
* @return {Array}
*/
Hash.prototype.values = function() {

	var array = [];
	this.each(function(_value, _key) {

		array.add(_value);
	});
	return array;
};

/**
* Returns true if this hash contains a mapping for the specified key.
* @param {Object} _key
* @return {Boolean]
*/
Hash.prototype.containsKey = function(_key) {

	return this.keys().contains(_key);
};

/**
* Returns true if this hash maps one or more keys to this value.
* @param {Object} _value
* @return {Boolean]
*/
Hash.prototype.containsValue = function(_value) {

	return this.values().contains(_value);
};

/**
* Returns true if this hash contains no key-value mappings.
* @return {Boolean}
*/
Hash.prototype.isEmpty = function() {

	return this.size() == 0;
};

/**
* Returns the number of key-value mappings in this hash.
* @return {Number}
*/
Hash.prototype.size = function() {

	return this.dataSize;
};

// mutating methods
/**
* Associates the specified value with the specified key in this hash.
* @param {Object} _key
* @param {Object} _value
*/
Hash.prototype.put = function(_key, _value) {

	if(_key && !_key.equals) { _key.equals = function(_object){ return this == _object; }; }
	if(_value && !_value.equals){ _value.equals = function(_object){ return this == _object; }; }

	var result = null;
	if(this.containsKey(_key)) {

		result = this.remove(_key);
	}

	this.data[_key] = _value;

	this.dataSize++;

	return result;
};

/**
* Removes and returns the mapping for this key from this hash if it is present.
* @param {Object} _key
* @return {Object}
*/
Hash.prototype.remove = function(_key) {

	var _index = this.keys().indexOf(_key);
	if(_index <= -1) {

		return null;
	}

	var _value = this.get(_key);

	delete this.data[_key];

	this.dataSize--;

	return _value;
};

/**
* Copies all of the mappings from the specified Hash to this Hash.
* @param {Map} _hash
*/
Hash.prototype.putAll = function(_hash) {

	var me = this;
	_hash.each(function(_value, _key) {

		me.put(_key, _value);
	});
};

/**
* Removes all mappings from this map.
*/
Hash.prototype.clear = function() {

	var me = this;
	this.keys().each(function(_value, _key) {

		me.remove(_value);
	});
};
