var TestCase = require("bn-unit").TestCase;

// setup test case
var EnumerableTest = function(){this.init.apply(this, arguments);};
exports.EnumerableTest = EnumerableTest;

EnumerableTest.prototype = new TestCase();
EnumerableTest.prototype.constructor = EnumerableTest;
EnumerableTest.parentclass = TestCase.prototype;
EnumerableTest.prototype.init = function() {};
EnumerableTest.prototype.setUp = function() {

	EnumerableTest.parentclass.setUp.apply(this, []);
};
EnumerableTest.prototype.tearDown = function() {

	EnumerableTest.parentclass.tearDown.apply(this, []);
};
EnumerableTest.getName = function() {

	return "Enumerable";
};

// add tests
EnumerableTest.prototype.testEach = function() {

	var result = 0;
	Number(1).toArray(5).each(function(_value) {

		result = _value;
	});
	this.assertEquals(5, result);
};

EnumerableTest.prototype.testEachBreak = function() {

	var result = 0;
	var array = [1, 2, 3, 4, 5];
	array.each(function(_value) {

		if(_value == 4) {

			array.$break();
		}

		result = _value;
	});
	this.assertEquals(3, result);
};

EnumerableTest.prototype.testEachContinue = function() {

	var result = 0;
	var array = [1, 2, 3, 4, 5];
	array.each(function(_value) {

		if(_value != 4) {

			array.$continue();
		}

		result = _value;
	});
	this.assertEquals(4, result);
};

EnumerableTest.prototype.testAll = function() {

	this.assertTrue(["ant", "bear", "cat"].all(function(_value){return _value.length >= 3}));
	this.assertFalse(["ant", "bear", "cat"].all(function(_value){return _value.length >= 4}));
	this.assertFalse([null, undefined, 99].all());
};

EnumerableTest.prototype.testAny = function() {

	this.assertTrue(["ant", "bear", "cat"].any(function(_value){return _value.length >= 3}));
	this.assertTrue(["ant", "bear", "cat"].any(function(_value){return _value.length >= 4}));
	this.assertTrue([null, undefined, 99].any());
};

EnumerableTest.prototype.testCollect = function() {

	this.assertTrue([1, 4, 9, 16].equals([1, 2, 3, 4].collect(function(_value){return _value*_value})));	
	this.assertTrue(["cat", "cat", "cat", "cat"].equals([1, 2, 3, 4].collect(function(_value){return "cat"})));
};

EnumerableTest.prototype.testDetect = function() {

	this.assertEquals(undefined, Number(1).toArray(10).detect(function(_value){return _value % 5 == 0 && _value % 7 == 0}));
	this.assertEquals(35, Number(1).toArray(100).detect(function(_value){return _value % 5 == 0 && _value % 7 == 0}));
};

EnumerableTest.prototype.testFindAll = function() {

	this.assertTrue([3, 6, 9].equals(Number(1).toArray(10).findAll(function(_value){return _value % 3 == 0})));
};

EnumerableTest.prototype.testGrep = function() {

	this.assertTrue(["Matthew", "Mathew", "Matt", "Mat"].equals(["Matthew", "Mathew", "Matt", "Mat"].grep(/Mat/)));
	this.assertTrue(["Matthew", "Matt"].equals(["Matthew", "Mathew", "Matt", "Mat"].grep(/Matt/)));
	this.assertTrue([3, 4].equals([1, 2, 3, 4, 5].grep(/3|4/)));
};

EnumerableTest.prototype.testInclude = function() {

	this.assertTrue(["Matthew", "Mathew", "Matt", "Mat"].include("Mat"));
	this.assertTrue(["Matthew", "Mathew", "Matt", "Mat"].include("Matt"));
	this.assertFalse(["Matthew", "Mathew", "Matt", "Mat"].include("Sam"));
	this.assertTrue([1, 2, 3, 4, 5].include(3))
};

EnumerableTest.prototype.testInject = function() {

	// Sum some numbers
	this.assertEquals(45, [5, 6, 7, 8, 9, 10].inject(0, function(_memo, _value){return _memo + _value}));
	// Multiply some numbers
	this.assertEquals(151200, [5, 6, 7, 8, 9, 10].inject(1, function(_memo, _value){return _memo * _value}));
	// find the longest word
	this.assertEquals("sheep", ["cat", "sheep", "bear"].inject("", function(_memo, _value){return _memo.length > _value.length ? _memo : _value}));
	// find the length of the longest word
	this.assertEquals(5, ["cat", "sheep", "bear"].inject(0, function(_memo, _value){return _memo >= _value.length ? _memo : _value.length}));
};

EnumerableTest.prototype.testInvoke = function() {

	this.assertTrue(["1", "1-2", "1-2-3", "1-2-3-4", "1-2-3-4-5"].equals([[1], [1, 2], [1, 2, 3], [1, 2, 3, 4], [1, 2, 3, 4, 5]].invoke("join", "-")));
};

EnumerableTest.prototype.testMax = function() {

	this.assertEquals("horse", ["albatross", "dog", "horse"].max());
	this.assertEquals("albatross", ["albatross", "dog", "horse"].max(function(a, _index, b){return a.length<b.length?-1:a.length>b.length?1:0}));
};

EnumerableTest.prototype.testMin = function() {

	this.assertEquals("albatross", ["albatross", "dog", "horse"].min());
	this.assertEquals("dog", ["albatross", "dog", "horse"].min(function(a, _index, b){return a.length<b.length?-1:a.length>b.length?1:0}));
};

EnumerableTest.prototype.testPartition = function() {

	this.assertTrue([[2,4,6],[1,3,5]].equals([1, 2, 3, 4, 5, 6].partition(function(_value){ return (_value%2 == 0) })));
};

EnumerableTest.prototype.testPluck = function() {

	this.assertTrue([1,2,3,4].equals([[1], [1,2], [1,2,3], [1,2,3,4]].pluck("length")));
};

EnumerableTest.prototype.testReject = function() {

	this.assertTrue([1, 2, 4, 5, 7, 8, 10].equals(Number(1).toArray(10).reject(function(_value){return (_value%3==0)})));
};

EnumerableTest.prototype.testSortBy = function() {

	this.assertTrue(["flea", "kea", "rhea"].equals(["rhea", "kea", "flea"].sortBy()));
	this.assertTrue([10, 9, 8, 7, 6, 5, 4, 3, 2, 1].equals(Number(1).toArray(10).sortBy(function(_value, _index){return -1 * _value})));
	this.assertTrue(["fig", "pear", "apple"].equals(["apple", "pear", "fig"].sortBy(function(_value, _index){return _value.length})));
};

EnumerableTest.prototype.testZip = function() {

	this.assertTrue([[1, 4, 7], [2, 5, 8], [3, 6, 9]].equals([1, 2, 3].zip([4, 5, 6], [7, 8, 9])));
	this.assertTrue([["cat", 1], ["dog", null]].equals(["cat", "dog"].zip([1])));
	this.assertTrue([[1], [2], [3]].equals([1, 2, 3].zip()));
};
