var TestCase = require("bn-unit").TestCase;
var Hash = require("bn-lang").Hash;

// setup test case
var HashTest = function(){this.init.apply(this, arguments);};
exports.HashTest = HashTest;

HashTest.prototype = new TestCase();
HashTest.prototype.constructor = HashTest;
HashTest.parentclass = TestCase.prototype;
HashTest.prototype.init = function() {}
HashTest.prototype.setUp = function(){

	HashTest.parentclass.setUp.apply(this, []);

	var equalsFunction = function(_object) {

		if(this === _object) {

			return true;
		}

		return this.name == _object.name;
	};

	var talkFunction = function(_word) {

		return _word +" "+this.name;
	};

	this.hash = new Hash();
	this.hash.put("One", {name:"One",id:1,toString:function(){return "One: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Two", {name:"Two",id:2,toString:function(){return "Two: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Three", {name:"Three",id:3,toString:function(){return "Three: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Four", {name:"Four",id:4,toString:function(){return "Four: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Five", {name:"Five",id:5,toString:function(){return "Five: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Six", {name:"Six",id:6,toString:function(){return "Six: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Seven", {name:"Seven",id:7,toString:function(){return "Seven: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Eight", {name:"Eight",id:8,toString:function(){return "Eight: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Nine", {name:"Nine",id:9,toString:function(){return "Nine: "+this.id;},equals:equalsFunction,talk:talkFunction});
	this.hash.put("Ten", {name:"Ten",id:10,toString:function(){return "Ten: "+this.id;},equals:equalsFunction,talk:talkFunction});
};
HashTest.prototype.tearDown = function() {

	HashTest.parentclass.tearDown.apply(this, []);

	this.hash = null;
};
HashTest.getName = function() {

	return "Hash";
};

// add tests

HashTest.prototype.testEquals = function() {

	this.assertTrue(this.hash.equals(this.hash));

	var newHash = new Hash();

	this.hash.each(function(_value, _key) {

		newHash.put(_key, _value);
	});

	this.assertTrue(this.hash.equals(newHash));

	var temp = newHash.get("Ten");
	newHash.remove("Ten");

	this.assertFalse(this.hash.equals(newHash));

	newHash.put("Ten", this.hash.get("One"));

	this.assertFalse(this.hash.equals(newHash));
};

// enumerable tests

HashTest.prototype.testEach = function() {

	var me = this;
	var counter = 0;
	this.hash.each(function(_value, _key) {

		counter++;
	});

	this.assertTrue(counter == 10);
};

HashTest.prototype.testEachBreak = function() {

	var me = this;
	var result = null;
	this.hash.each(function(_value, _key) {

		if(_key == "Seven") {

			result = _value;
			me.hash.$break();
		}
	});

	this.assertTrue(result.name == "Seven");
};

HashTest.prototype.testEachContinue = function() {

	var me = this;
	var result = null;
	var counter = 0;
	var counter2 = 0;
	this.hash.each(function(_value, _key) {

		counter++;

		if(_key != "Seven") {

			me.hash.$continue();
		}

		result = _value;
		counter2++;
	});

	this.assertTrue(result.name == "Seven");
	this.assertTrue(counter == 10);
	this.assertTrue(counter2 == 1);
};

HashTest.prototype.testAll = function() {

	var result = this.hash.all(function(_value, _key) {

		return _value.id >= 1 && _value.id <= 10;
	});

	this.assertTrue(result);

	var result = this.hash.all(function(_value, _key) {

		return _value.id >= 1 && _value.id <= 9;
	});

	this.assertFalse(result);
};

HashTest.prototype.testAny = function() {

	var result = this.hash.any(function(_value, _key) {

		return _value.id == 5;
	});

	this.assertTrue(result);

	var result = this.hash.any(function(_value, _key) {

		return _value.id == 11;
	});

	this.assertFalse(result);
};

HashTest.prototype.testCollect = function() {

	var items = this.hash.collect(function(_value, _key) {

		return _value.name;
	});
	this.assertTrue(items[0] == "One");
	this.assertTrue(items[1] == "Two");
	this.assertTrue(items[2] == "Three");
	this.assertTrue(items[3] == "Four");
	this.assertTrue(items[4] == "Five");
	this.assertTrue(items[5] == "Six");
	this.assertTrue(items[6] == "Seven");
	this.assertTrue(items[7] == "Eight");
	this.assertTrue(items[8] == "Nine");
	this.assertTrue(items[9] == "Ten");
	this.assertTrue(items.length == 10);

	var items = this.hash.collect()
	this.assertTrue(items[0].name == "One");
	this.assertTrue(items[1].name == "Two");
	this.assertTrue(items[2].name == "Three");
	this.assertTrue(items[3].name == "Four");
	this.assertTrue(items[4].name == "Five");
	this.assertTrue(items[5].name == "Six");
	this.assertTrue(items[6].name == "Seven");
	this.assertTrue(items[7].name == "Eight");
	this.assertTrue(items[8].name == "Nine");
	this.assertTrue(items[9].name == "Ten");
	this.assertTrue(items.length == 10);
};

HashTest.prototype.testDetect = function() {

	var result = this.hash.detect(function(_value, _key) {

		return _value.name == "Eight";
	});

	this.assertTrue(result.name == "Eight");
};

HashTest.prototype.testFindAll = function() {

	var items = this.hash.findAll(function(_value, _key) {
	
		return _value.id >= 6 && _value.id <= 10;
	});

	this.assertTrue(items[0].name == "Six");
	this.assertTrue(items[1].name == "Seven");
	this.assertTrue(items[2].name == "Eight");
	this.assertTrue(items[3].name == "Nine");
	this.assertTrue(items[4].name == "Ten");
};

HashTest.prototype.testGrep = function() {

	var items = this.hash.grep(/ne/, function(_value, _key) {

		return _value.id;
	});

	this.assertTrue(items[0] == "1");
	this.assertTrue(items[1] == "9");
};

HashTest.prototype.testInclude = function() {

	var match = this.hash.get("Five");
	this.assertTrue(this.hash.include(match));

	var match = {name:"Eleven"};
	this.assertFalse(this.hash.include(match));
};

HashTest.prototype.testInject = function() {

	var memo = this.hash.inject("Zero ", function(_memo, _value, _key) {

		return _memo + _value.name + " ";
	});

	this.assertTrue(memo == "Zero One Two Three Four Five Six Seven Eight Nine Ten ");
};

HashTest.prototype.testInvoke = function() {

	this.hash.invoke("talk", "My name is ");
};

HashTest.prototype.testMax = function() {

	var result = this.hash.max(function(_value, _key, _value2) {

		if(_value.id < _value2.id) {

			return _value2;
		}
		return _value.id;
	});

	this.assertSame(this.hash.get("Ten"), result);
};

HashTest.prototype.testMin = function() {

	var result = this.hash.min(function(_value, _key) {

		return _value.id;
	});

	this.assertSame(this.hash.get("One"), result);
};

HashTest.prototype.testPartition = function() {

	var items = this.hash.partition(function(_value, _key) {

		if(_value.id >= 1 && _value.id <= 5) {

			return true;
		}

		return false
	});

	var trues = items[0];
	var falses = items[1];

	this.assertTrue(trues.length == 5);
	this.assertTrue(falses.length == 5);

	this.assertTrue(trues[0].name == "One");
	this.assertTrue(trues[1].name == "Two");
	this.assertTrue(trues[2].name == "Three");
	this.assertTrue(trues[3].name == "Four");
	this.assertTrue(trues[4].name == "Five");

	this.assertTrue(falses[0].name == "Six");
	this.assertTrue(falses[1].name == "Seven");
	this.assertTrue(falses[2].name == "Eight");
	this.assertTrue(falses[3].name == "Nine");
	this.assertTrue(falses[4].name == "Ten");
};

HashTest.prototype.testPluck = function() {

	var items = this.hash.pluck("name");

	this.assertTrue(items[0] == "One");
	this.assertTrue(items[1] == "Two");
	this.assertTrue(items[2] == "Three");
	this.assertTrue(items[3] == "Four");
	this.assertTrue(items[4] == "Five");
	this.assertTrue(items[5] == "Six");
	this.assertTrue(items[6] == "Seven");
	this.assertTrue(items[7] == "Eight");
	this.assertTrue(items[8] == "Nine");
	this.assertTrue(items[9] == "Ten");
};

HashTest.prototype.testReject = function() {

	var items = this.hash.reject(function(_value, _key) {

		return _value.id < 9;
	});

	this.assertTrue(items[0].name == "Nine");
	this.assertTrue(items[1].name == "Ten");
};

HashTest.prototype.testSortBy = function() {

	var items = this.hash.toArray();
	this.assertTrue(items[0].id == 1);

	var items2 = this.hash.sortBy(function(_value, _key) {

		return -1 * _value.id;
	});
	this.assertTrue(items2[0].id == 10);
};

HashTest.prototype.testZip = function() {

	var map1 = new Hash();
	var map2 = new Hash();
	var map3 = new Hash();

	map1.put(0, this.hash.get("One"));
	map1.put(1, this.hash.get("Two"));
	map1.put(2, this.hash.get("Three"));

	map2.put(0, this.hash.get("Four"));
	map2.put(1, this.hash.get("Five"));
	map2.put(2, this.hash.get("Six"));

	map3.put(0, this.hash.get("Seven"));
	map3.put(1, this.hash.get("Eight"));
	map3.put(2, this.hash.get("Nine"));
	
	this.assertTrue([[this.hash.get("One"), this.hash.get("Four"), this.hash.get("Seven")],
						[this.hash.get("Two"), this.hash.get("Five"), this.hash.get("Eight")],
						[this.hash.get("Three"), this.hash.get("Six"), this.hash.get("Nine")]].equals(map1.zip(map2, map3)));
};

// querying tests

HashTest.prototype.testGet = function() {

	this.hash.put(null, 101010101);

	this.assertTrue(this.hash.get(null) != null);
	this.assertTrue(this.hash.get("One") != null);
	this.assertTrue(this.hash.get("Two") != null);
	this.assertTrue(this.hash.get("Three") != null);
	this.assertTrue(this.hash.get("Four") != null);
	this.assertTrue(this.hash.get("Five") != null);
	this.assertTrue(this.hash.get("Six") != null);
	this.assertTrue(this.hash.get("Seven") != null);
	this.assertTrue(this.hash.get("Eight") != null);
	this.assertTrue(this.hash.get("Nine") != null);
	this.assertTrue(this.hash.get("Ten") != null);

	this.assertTrue(this.hash.get("Eleven") == null);
};

HashTest.prototype.testKeySet = function() {

	var keys = this.hash.keys();
	this.assertTrue(keys.size() == 10);

	this.assertTrue(keys.contains("One"));
	this.assertTrue(keys.contains("Two"));
	this.assertTrue(keys.contains("Three"));
	this.assertTrue(keys.contains("Four"));
	this.assertTrue(keys.contains("Five"));
	this.assertTrue(keys.contains("Six"));
	this.assertTrue(keys.contains("Seven"));
	this.assertTrue(keys.contains("Eight"));
	this.assertTrue(keys.contains("Nine"));
	this.assertTrue(keys.contains("Ten"));

	this.assertFalse(keys.contains("Eleven"));
};

HashTest.prototype.testValues = function() {

	var values = this.hash.values();
	this.assertTrue(values.size() == 10);
};

HashTest.prototype.testContainsKey = function() {

	var me = this;
	this.hash.keys().each(function(_value) {

		me.assertTrue(me.hash.containsKey(_value));
	});

	this.assertFalse(this.hash.containsKey("Eleven"));
};

HashTest.prototype.testContainsValue = function() {

	var me = this;
	this.hash.values().each(function(_value) {

		me.assertTrue(me.hash.containsValue(_value));
	});
};

HashTest.prototype.testIsEmpty = function() {

	this.assertFalse(this.hash.isEmpty());
	this.hash.clear();
	this.assertTrue(this.hash.isEmpty());
};

HashTest.prototype.testSize = function() {

	this.assertTrue(this.hash.size() == 10);
	this.hash.clear();
	this.assertTrue(this.hash.size() == 0);
};

// mutating tests

HashTest.prototype.testPut = function() {

	this.assertTrue(this.hash.size() == 10);

	var key = "Test";
	var object = 101010101;

	this.hash.put(key, object);

	this.assertTrue(this.hash.size() == 11);
	this.assertTrue(this.hash.get(key).equals(object));

	this.hash.put(null, object);
	this.assertTrue(this.hash.size() == 12);
	this.assertTrue(this.hash.get(null).equals(object));
};

HashTest.prototype.testRemove = function() {

	this.assertTrue(this.hash.size() == 10);
	this.hash.remove("One");
	this.assertTrue(this.hash.size() == 9);
	this.hash.remove("Two");
	this.assertTrue(this.hash.size() == 8);
	this.hash.remove("Three");
	this.assertTrue(this.hash.size() == 7);
	this.hash.remove("Four");
	this.assertTrue(this.hash.size() == 6);
	this.hash.remove("Five");
	this.assertTrue(this.hash.size() == 5);
	this.hash.remove("Six");
	this.assertTrue(this.hash.size() == 4);
	this.hash.remove("Seven");
	this.assertTrue(this.hash.size() == 3);
	this.hash.remove("Eight");
	this.assertTrue(this.hash.size() == 2);
	this.hash.remove("Nine");
	this.assertTrue(this.hash.size() == 1);
	this.hash.remove("Ten");
	this.assertTrue(this.hash.size() == 0);
};

HashTest.prototype.testPutAll = function() {

	var key1 = "Annie";
	var key2 = "Matt";
	var key3 = "Sam";

	var newHash = new Hash();
	newHash.put(key1, 28);
	newHash.put(key2, 30);
	newHash.put(key3, 1);

	this.assertTrue(this.hash.size() == 10);

	this.hash.putAll(newHash);

	this.assertTrue(this.hash.size() == 13);

	this.assertTrue(this.hash.get(key1).equals(newHash.get(key1)));
	this.assertTrue(this.hash.get(key2).equals(newHash.get(key2)));
	this.assertTrue(this.hash.get(key3).equals(newHash.get(key3)));
};

HashTest.prototype.testClear = function() {

	this.assertTrue(this.hash.size() == 10);
	this.hash.clear();
	this.assertTrue(this.hash.size() == 0);
};
