var TestCase = require("bn-unit").TestCase;

// setup test case
var StringTest = function(){this.init.apply(this, arguments);};
exports.StringTest = StringTest;

StringTest.prototype = new TestCase();
StringTest.prototype.constructor = StringTest;
StringTest.parentclass = TestCase.prototype;
StringTest.prototype.init = function() {}
StringTest.prototype.setUp = function() {

	StringTest.parentclass.setUp.apply(this, []);
   	this.str = "This is a test";
};
StringTest.prototype.tearDown = function() {

	StringTest.parentclass.tearDown.apply(this, []);
	this.str = null;
};
StringTest.getName = function() {

	return "String";
};

// add tests
StringTest.prototype.testEach = function() {

	var me = this;
	var part1, part2, part3, part4;
	this.str.each(" ", function(_value, _index) {

		switch(_index) {

			case 0:
				part1 = _value;
				break;
			case 1:
				part2 = _value;
				break;
			case 2:
				part3 = _value;
				break;
			case 3:
				part4 = _value;
				break;
			default:
				me.fail("could not handle index: " + _index + " with value: " + _value);
		}
	});

	this.assertTrue("This".equals(part1));
	this.assertTrue("is".equals(part2));
	this.assertTrue("a".equals(part3));
	this.assertTrue("test".equals(part4));
};

StringTest.prototype.testTrim = function() {

	this.str = "    space    ";
	this.assertTrue("space".equals(this.str.trim()));
};

StringTest.prototype.testEscapeXml = function() {

	this.str = "this is < that and > that so & you and take \" and take \'";
	this.assertTrue(this.str.indexOf("&") > -1);
	this.assertTrue(this.str.indexOf("<") > -1);
	this.assertTrue(this.str.indexOf(">") > -1);
	this.assertTrue(this.str.indexOf('"') > -1);
	this.assertTrue(this.str.indexOf("'") > -1);

	this.str = this.str.escapeXml();
	this.assertTrue(this.str.indexOf("& ") == -1);
	this.assertTrue(this.str.indexOf("<") == -1);
	this.assertTrue(this.str.indexOf(">") == -1);
	this.assertTrue(this.str.indexOf('"') == -1);
	this.assertTrue(this.str.indexOf("'") == -1);
};

StringTest.prototype.testUnescapeXml = function() {

	this.str = "this is &lt; that and &gt; that so &amp; you and take &quot; and take &apos;";
	this.assertTrue(this.str.indexOf("& ") == -1);
	this.assertTrue(this.str.indexOf("<") == -1);
	this.assertTrue(this.str.indexOf(">") == -1);
	this.assertTrue(this.str.indexOf('"') == -1);
	this.assertTrue(this.str.indexOf("'") == -1);

	this.str = this.str.unescapeXml();
	this.assertTrue(this.str.indexOf("&") > -1);
	this.assertTrue(this.str.indexOf("<") > -1);
	this.assertTrue(this.str.indexOf(">") > -1);
	this.assertTrue(this.str.indexOf('"') > -1);
	this.assertTrue(this.str.indexOf("'") > -1);
};

StringTest.prototype.testStartsWith = function() {

	this.assertTrue(this.str.startsWith("This i"));
	this.assertFalse(this.str.startsWith(" test"));
};

StringTest.prototype.testEndsWith = function() {

	this.assertTrue(this.str.endsWith(" test"));
	this.assertFalse(this.str.endsWith("This i"));
};

StringTest.prototype.testReverse = function() {

	this.assertTrue("tset a si sihT".equals(this.str.reverse()));
};

StringTest.prototype.testTruncate = function() {

	this.assertTrue("This ".equals(this.str.truncate(5)));
};

StringTest.prototype.testCapitalize = function() {

	this.assertTrue(this.str.equals("this is a test".capitalize()));
};

StringTest.prototype.testCamelCase = function() {

	this.assertTrue("backgroundImage".equals("background-image".camelCase()));
	this.assertTrue("backgroundImage".equals("-background-image".camelCase()));
	this.assertTrue("backgroundImage".equals("background-image-".camelCase()));
	this.assertTrue("backgroundImage".equals("-background-image-".camelCase()));

	this.assertTrue("backgroundImage".equals("background_image".camelCase()));
	this.assertTrue("backgroundImage".equals("_background_image".camelCase()));
	this.assertTrue("backgroundImage".equals("background_image_".camelCase()));
	this.assertTrue("backgroundImage".equals("_background_image_".camelCase()));
};

StringTest.prototype.testHyphenCase = function() {

	this.assertTrue("background-image".equals("backgroundImage".hyphenCase()));
};

StringTest.prototype.testSnakeCase = function() {

	this.assertTrue("background_image".equals("backgroundImage".snakeCase()));
};

StringTest.prototype.testIsUpperCase = function() {

	this.assertTrue("THIS IS A TEST".isUpperCase());
	this.assertFalse(this.str.isUpperCase());

	this.assertTrue("A".isUpperCase());
	this.assertFalse("a".isUpperCase());
};

StringTest.prototype.testIsLowerCase = function() {

	this.assertTrue("this is a test".isLowerCase());
	this.assertFalse(this.str.isLowerCase());

	this.assertFalse("A".isLowerCase());
	this.assertTrue("a".isLowerCase());
};

StringTest.prototype.testIsBlank = function() {

	this.assertTrue("".isBlank());
	this.assertTrue(" ".isBlank());
	this.assertTrue("      ".isBlank());
	this.assertTrue("\n".isBlank());
	this.assertTrue("\t".isBlank());
	this.assertTrue("\r".isBlank());
	this.assertFalse("    a    ".isBlank());
};

StringTest.prototype.testIsNumeric = function() {

	this.assertFalse("".isNumeric());

	this.assertTrue("0123456789".isNumeric());
	this.assertTrue("23459082734502983745203984752093857290475".isNumeric());

	this.assertFalse("a213421342".isNumeric());
	this.assertFalse("2134a21342".isNumeric());
	this.assertFalse("213421342a".isNumeric());
};
