/**
* @class AssertionFailedError
* @constructor
* @param {String} _message
*/
var AssertionFailedError = function(){this.init.apply(this, arguments);};
exports.AssertionFailedError = AssertionFailedError;

/** @ignore */
AssertionFailedError.prototype.init = function(_message) {

	this.message = _message;
};
/**
* @class TestFailure
* @constructor
*/
var TestFailure = function(){this.init.apply(this, arguments);};
exports.TestFailure = TestFailure;

TestFailure.prototype = {
	/** @ignore */
	init : function(_test, _testCase, _exception) {

		this.test = _test;
		this.testCase = _testCase;
		this.exception = _exception;
	},
	isFailure : function() {

		return this.exception.constructor == AssertionFailedError
	},
	getTest : function() {

		return this.test;
	},
	getTestCase : function() {

		return this.testCase;
	},
	getException : function() {

		return this.exception;
	}
};
/**
* @class ResultPrinterText
* @constructor
* @param {Object} _output
*/
var ResultPrinterText = function(){this.init.apply(this, arguments);};
exports.ResultPrinterText = ResultPrinterText;

ResultPrinterText.prototype = {
	/** @ignore */
	init : function(_output) {

		this.output = _output;
	},
	print : function(_test, _result, _runTime) {

		var _text = "";

		_text += "[unit] Running ";
		_text += _test.getName();

		_text += "\n[unit] Tests run: ";
		_text += _result.runCount();

		_text += ", Failures: ";
		_text += _result.failureCount();

		_text += ", Errors: ";
		_text += _result.errorCount();

		_text += ", Time elapsed: ";
		_text += _runTime;
		_text += " millisec";

		if(!_result.wasSuccessful()) {

			_text += "\n[unit] Test ";
			_text += _test.getName();
			_text += " FAILED";

			if(_result.failureCount() > 0) {

				_text += "\n\n\t\tFailures";
				
				var _failures = _result.getFailures();
				for (var i=0;i<_failures.length;i++) {

					var _failure = _failures[i];

					_text += "\n\t\t";
					_text += _failure.getTestCase();
					_text += ": ";
					_text += _failure.getTest();
					_text += " - ";
					_text += _failure.getException().message;
				}
			}

			if(_result.errorCount() > 0) {

				_text += "\n\n\t\tErrors";

				var _errors = _result.getErrors();
				for (var i=0;i<_errors.length;i++) {

					var _error = _errors[i];

					_text += "\n\t\t";
					_text += _error.getTestCase();
					_text += ": ";
					_text += _error.getTest();
					_text += " - ";
					_text += _error.getException().message;
				}
			}
		}

		_text += "\n";

		this.output(_text);
	},
	startTest : function(_test) {},
	endTest : function(_test) {},
	addSuccess : function(_test) {},
	addFailure : function(_test, _failure) {},
	addError : function(_test, _error) {}
};
/**
* @class ResultPrinterHTML
* @constructor
* @param {HTMLElement} _container
*/
var ResultPrinterHTML = function(){this.initialize.apply(this, arguments);};
exports.ResultPrinterHTML = ResultPrinterHTML;

ResultPrinterHTML.prototype = {
	/** @ignore */
	initialize : function(_container) {

		this.runCounter = 1;

		this.container = _container;
		this.document = this.container.ownerDocument;

		this.summaryTests = this.document.createElement("div");

		this.formatException = function(_exception) {

			if(_exception) {

				var _excepContainer = this.document.createElement("span");

				var _exString = "Exception: ";
				if(_exception.message) {

					_exString += _exception.message;

				} else if(_exception.description) {

					_exString += _exception.description;
				}
				if(_exception.lineNumber) {

					_exString += " on line number ";
					_exString += _exception.lineNumber;
				}
				if(_exception.fileName) {

					_exString += " in file ";
					_exString += _exception.fileName;
				}

				_excepContainer.appendChild(this.document.createTextNode(_exString));

				return _excepContainer;
			}
			return this.document.createTextNode("");
		};
	},
	print : function(_test, _result, _runTime) {

		var me = this;

		// title
		var titleContainer = this.document.createElement("h3");
		var title = this.document.createTextNode(_test.getName());
		titleContainer.appendChild(title);
		this.container.appendChild(titleContainer);

		// summary of tests
		this.container.appendChild(this.summaryTests);
		this.container.appendChild(this.document.createElement("br"));

		// summary text
		var summaryText = this.document.createTextNode(_result.runCount() + " tests ran in " + _runTime / 1000.0 + " seconds");
		this.container.appendChild(summaryText);
		this.container.appendChild(this.document.createElement("br"));

		// errors and failures
		if(!_result.wasSuccessful()) {

			if(_result.failureCount() > 0) {

				this.container.appendChild(this.document.createElement("br"));
				var failureTitleContainer = this.document.createElement("div");
				var failureTitle = this.document.createTextNode("Failures");
				failureTitleContainer.appendChild(failureTitle);
				failureTitleContainer.style.fontWeight = 'bold';
				var failureContainer = this.document.createElement("div");

				var _failures = _result.getFailures();
				for (var i=0;i<_failures.length;i++) {

					var _failure = _failures[i];

					failureContainer.appendChild(this.document.createTextNode(_failure.getTestCase()+": "+_failure.getTest() +" - "+ _failure.getException().message));
					failureContainer.appendChild(this.document.createElement('br'));
				}

				failureContainer.style.color = 'red';
				this.container.appendChild(failureTitleContainer);
				this.container.appendChild(failureContainer);
			}

			if(_result.errorCount() > 0) {

				this.container.appendChild(this.document.createElement("br"));
				var errorTitleContainer = this.document.createElement("div");
				var errorTitle = this.document.createTextNode("Errors");
				errorTitleContainer.appendChild(errorTitle);
				errorTitleContainer.style.fontWeight = 'bold';
				var errorContainer = this.document.createElement("div");

				var _errors = _result.getErrors();
				for (var i=0;i<_errors.length;i++) {

					var _error = _errors[i];

					errorContainer.appendChild(this.document.createTextNode(_error.getTestCase()+": "+_error.getTest() +" - "));
					errorContainer.appendChild(me.formatException(_error.getException()));
					errorContainer.appendChild(this.document.createElement('br'));
				}

				errorContainer.style.color = 'red';
				this.container.appendChild(errorTitleContainer);
				this.container.appendChild(errorContainer);
			}

			this.container.appendChild(this.document.createElement("br"));
		}

		// status bar
		var statusBar = this.document.createElement("div");
		var statusText = this.document.createTextNode(_result.wasSuccessful()?"OK!":"NOT OK!");
		statusBar.appendChild(statusText);
		statusBar.style.backgroundColor = _result.wasSuccessful()?"lightgreen":"red";
		statusBar.style.color = 'black';
		statusBar.style.fontWeight = 'bold';
		statusBar.style.textAlign = 'center';
		statusBar.style.marginTop = '1em';
		this.container.appendChild(statusBar);
	},
	startTest : function(_test) {},
	endTest : function(_test) {},
	addSuccess : function(_test) { this.printResult("+"); },
	addFailure : function(_test, _failure) { this.printResult("F"); },
	addError : function(_test, _error) { this.printResult("E"); },
	printResult : function(_char) {

		this.summaryTests.appendChild(this.document.createTextNode(_char));
		if(this.runCounter >= 100) {

			this.runCounter = 1;
			this.summaryTests.appendChild(this.document.createElement("br"));

		} else {

			this.runCounter++;
		}
	}
};
/**
* @class TestResult
* @constructor
*/
var TestResult = function(){this.init.apply(this, arguments);};
exports.TestResult = TestResult;

TestResult.prototype = {
	/** @ignore */
	init : function() {

		this.runTests = 0;
		this.successes = [];
		this.failures = [];
		this.errors = [];
		this.listeners = [];
	},
	addListener : function(_listener) {

		this.listeners[this.listeners.length] = _listener;
	},
	removeListener : function(_listener) {

		var _temp = [];

		for(var i=0;i<this.listeners.length;i++) {

			if(!this.listeners[i] == _listener) {

				_temp[_temp.length] = _listener;
			}
		}

		this.listeners = _temp;
	},
	runCount : function() {

		return this.runTests;
	},
	successCount : function() {

		return this.successes.length();
	},
	getSuccesses : function() {

		return this.successes;
	},
	addSuccess : function(_test) {

		this.successes[this.successes.length] = _test;

		for(var i=0;i<this.listeners.length;i++) {

			this.listeners[i].addSuccess(_test);
		}
	},
	failureCount : function() {

		return this.failures.length;
	},
	getFailures : function() {

		return this.failures;
	},
	addFailure : function(_test, _testCaseName, _exception) {

		this.failures[this.failures.length] = new TestFailure(_test, _testCaseName, _exception);

		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].addFailure(_test, _testCaseName, _exception);
		}
	},
	errorCount : function() {

		return this.errors.length;
	},
	getErrors : function() {

		return this.errors;
	},
	addError : function(_test, _testCaseName, _exception) {

		this.errors[this.errors.length] = new TestFailure(_test, _testCaseName, _exception);
		
		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].addError(_test, _testCaseName, _exception);
		}
	},
	startTest : function(_test) {

		this.runTests += _test.countTestCases();

		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].startTest(_test);
		}
	},
	endTest : function(_test) {

		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].endTest(_test);
		}
	},
	wasSuccessful : function() {

		return this.failureCount() == 0 && this.errorCount() == 0;
	},
	run : function(_test) {

		this.startTest(_test);
	
		var _success = true;
		try {

			_test.runBare();

		} catch(_exception) {

			_success = false;

			if(_exception.constructor == AssertionFailedError) {

				this.addFailure(_test, _test.constructor.getName(), _exception);

			} else {

				this.addError(_test, _test.constructor.getName(), _exception);
			}
		}

		if(_success) {

			this.addSuccess(_test);
		}

		this.endTest(_test);
	}
};
/**
* @class TestSuite
* @constructor
* @param {String} _name
*/
var TestSuite = function(){this.init.apply(this, arguments);};
exports.TestSuite = TestSuite;

TestSuite.prototype = {
	/** @ignore */
	init : function(_name, _test) {

		this.name = _name;
		this.testClass = _test;
		this.tests = [];

		if(_test) {

			this.__getTests__(_test);
		}
	},	
	__getTests__ : function(_test) {

		var parentClass = _test;
		var names = [];
		while(true) {

			var methods = [];
			for(var prop in parentClass.prototype) {

				if(typeof parentClass.prototype[prop] == "function") {

					methods[methods.length] = prop;
				}
			}

			for (var i=0;i<methods.length;i++) {

				if(this.__isTestMethod__(methods[i])) {

					this.__addTestMethod__(methods[i], names, _test);
				}
			}

			parentClass = parentClass.parentclass;
			if(!parentClass) {

				break;
			}
		}
	},
	__isTestMethod__ : function(_method) {

		return _method.indexOf("test") == 0;
	},
	__addTestMethod__ : function(_method, _names, _test) {

		for(var i=0;i<_names.length;i++) {

			if(_names[i] == _method) {

				return;
			}
		}

		_names[_names.length] = _method;

		var test = new _test();
		test.setName(_method);

		this.addTest(test);
	},
	getName : function() {

		return this.name;
	},
	getTestClass : function() {

		return this.testClass;
	},
	countTestCases : function() {

		var count = 0;

		for (var i=0;i<this.tests.length;i++) {

			count += this.tests[i].countTestCases();
		}

		return count;
	},
	run : function(_result) {

		var result = null;
		if(_result) {

			result = _result;

		} else {

			result = new TestResult();
		}

		for (var i=0;i<this.tests.length;i++) {

			this.runTest(this.tests[i], result);
		}
	},
	runTest : function(_test, _result) {

		_test.run(_result);
	},
	addTest : function(_test) {

		this.tests[this.tests.length] = _test;
	},
	addTestSuite : function(_testClass) {

		this.addTest(_testClass.suite());
	},
	testAt : function(_index) {

		return this.tests[_index];
	},
	testCount : function() {

		return this.tests.length;
	},
	getTests : function() {

		return this.tests;
	}

};
/**
* @class TestRunner
* @constructor
* @param {ResultPrinter} _resultPrinter
*/
var TestRunner = function(){this.init.apply(this, arguments);};
exports.TestRunner = TestRunner;

TestRunner.prototype = {
	/** @ignore */
	init : function(_resultPrinter) {

		this.resultPrinter = _resultPrinter;
	},
	getTest : function(_testSuiteClass) {

		return _testSuiteClass.suite();
	},
	runTest : function(_test) {

		var _suite = new TestSuite("Tests for "+_test.getName(), _test);

		this.run(_suite);
	},
	runSuite : function(_testSuite) {

		var _suite = this.getTest(_testSuite);

		this.run(_suite);
	},
	run : function(_test) {

		var result = new TestResult();

		result.addListener(this.resultPrinter);

		var startTime = new Date().getTime();

		_test.run(result);

		var endTime = new Date().getTime();

		var runTime = endTime - startTime;

		this.resultPrinter.print(_test, result, runTime);

		return result;
	}
};
/**
* @class Assert
*/
var Assert = function(){this.init.apply(this, arguments);};
exports.Assert = Assert;

/** @ignore */
Assert.prototype.init = function(){};

Assert.assertTrue = function(_condition, _message) {

	if(!_condition) {

		if(!_message){ _message = "Assert True failed"; }
		this.failTrue(_message);
	}
};

Assert.assertFalse = function(_condition, _message) {

	if(!_message){ _message = "Assert False failed"; }
	this.assertTrue(!_condition, _message);
};

Assert.assertEquals = function(_expected, _actual, _message) {

	if(_expected == null && _actual == null) {

		return;
	}

	if(_expected != null && _expected == _actual) {
		return;
	}

	this.failNotEquals(_expected, _actual, _message);
};

Assert.assertNull = function(_object, _message) {

	this.assertTrue((_object == null), _message);
};

Assert.assertNotNull = function(_object, _message) {

	this.assertTrue((_object != null), _message);
};

Assert.assertSame = function(_expected, _actual, _message) {

	if(_expected === _actual) {

		return;
	}
	this.failNotSame(_expected, _actual, _message);
};

Assert.assertNotSame = function(_expected, _actual, _message) {

	if(_expected === _actual) {

		this.failSame(_message);
	}
};

Assert.fail = function(_message) {

	throw new AssertionFailedError(_message);
};

Assert.failTrue = function(_message) {

	this.fail(_message);
};

Assert.failNotEquals = function(_expected, _actual, _message) {

	var formatted = "";
	if(_message) {

		formatted = _message + " ";
	}
	this.fail(formatted + "expected:< " + _expected + " > but was:< " + _actual + " >");
};

Assert.failSame = function(_message) {

	var formatted = "";
	if(_message) {

		formatted = _message + " ";
	}
	this.fail(formatted + "expected not same");
};

Assert.failNotSame = function(_expected, _actual, _message) {

	var formatted = "";
	if(_message) {

		formatted = _message + " ";
	}
	this.fail(formatted + "expected same:< " + _expected + " > was not:< " + _actual + " >");
};
/**
* @class TestCase
* @constructor
*/
var TestCase = function(){this.init.apply(this, arguments);};
exports.TestCase = TestCase;

TestCase.prototype = {
	/** @ignore */
	init : function() {

		this.name = null;
	},
	setUp : function(){},
	tearDown : function(){},
	getName : function() {

		return this.name;
	},
	setName : function(_name) {

		this.name = _name;
	},
	countTestCases : function() {

		return 1;
	},
	run : function(_result) {

		if(_result) {

			_result.run(this);
			
		} else {

			var result = new TestResult();
			result.run(this);
		}
	},
	runBare : function() {

		this.setUp();
		try {

			this.runTest();

		} finally {

			this.tearDown();
		}
	},
	runTest : function() {

		this.assertNotNull(this.name, "TestCase.name cannot be null");

		var runMethod = this[this.name];

		if(runMethod) {

			runMethod.apply(this, []);

		} else {

			this.fail("Method \""+this.name+"\" not found");
		}
	},
	toString : function() {

		return this.name;
	},
	assertTrue : Assert.assertTrue,
	assertFalse : Assert.assertFalse,
	assertEquals : Assert.assertEquals,
	assertNull : Assert.assertNull,
	assertNotNull : Assert.assertNotNull,
	assertSame : Assert.assertSame,
	assertNotSame : Assert.assertNotSame,
	fail : Assert.fail,
	failTrue : Assert.failTrue,
	failNotEquals : Assert.failNotEquals,
	failSame : Assert.failSame,
	failNotSame : Assert.failNotSame
};
