/**
* @class ResultPrinterHTML
* @constructor
* @param {HTMLElement} _container
*/
var ResultPrinterHTML = function(){this.initialize.apply(this, arguments);};
exports.ResultPrinterHTML = ResultPrinterHTML;

ResultPrinterHTML.prototype = {
	/** @ignore */
	initialize : function(_container) {

		this.runCounter = 1;

		this.container = _container;
		this.document = this.container.ownerDocument;

		this.summaryTests = this.document.createElement("div");

		this.formatException = function(_exception) {

			if(_exception) {

				var _excepContainer = this.document.createElement("span");

				var _exString = "Exception: ";
				if(_exception.message) {

					_exString += _exception.message;

				} else if(_exception.description) {

					_exString += _exception.description;
				}
				if(_exception.lineNumber) {

					_exString += " on line number ";
					_exString += _exception.lineNumber;
				}
				if(_exception.fileName) {

					_exString += " in file ";
					_exString += _exception.fileName;
				}

				_excepContainer.appendChild(this.document.createTextNode(_exString));

				return _excepContainer;
			}
			return this.document.createTextNode("");
		};
	},
	print : function(_test, _result, _runTime) {

		var me = this;

		// title
		var titleContainer = this.document.createElement("h3");
		var title = this.document.createTextNode(_test.getName());
		titleContainer.appendChild(title);
		this.container.appendChild(titleContainer);

		// summary of tests
		this.container.appendChild(this.summaryTests);
		this.container.appendChild(this.document.createElement("br"));

		// summary text
		var summaryText = this.document.createTextNode(_result.runCount() + " tests ran in " + _runTime / 1000.0 + " seconds");
		this.container.appendChild(summaryText);
		this.container.appendChild(this.document.createElement("br"));

		// errors and failures
		if(!_result.wasSuccessful()) {

			if(_result.failureCount() > 0) {

				this.container.appendChild(this.document.createElement("br"));
				var failureTitleContainer = this.document.createElement("div");
				var failureTitle = this.document.createTextNode("Failures");
				failureTitleContainer.appendChild(failureTitle);
				failureTitleContainer.style.fontWeight = 'bold';
				var failureContainer = this.document.createElement("div");

				var _failures = _result.getFailures();
				for (var i=0;i<_failures.length;i++) {

					var _failure = _failures[i];

					failureContainer.appendChild(this.document.createTextNode(_failure.getTestCase()+": "+_failure.getTest() +" - "+ _failure.getException().message));
					failureContainer.appendChild(this.document.createElement('br'));
				}

				failureContainer.style.color = 'red';
				this.container.appendChild(failureTitleContainer);
				this.container.appendChild(failureContainer);
			}

			if(_result.errorCount() > 0) {

				this.container.appendChild(this.document.createElement("br"));
				var errorTitleContainer = this.document.createElement("div");
				var errorTitle = this.document.createTextNode("Errors");
				errorTitleContainer.appendChild(errorTitle);
				errorTitleContainer.style.fontWeight = 'bold';
				var errorContainer = this.document.createElement("div");

				var _errors = _result.getErrors();
				for (var i=0;i<_errors.length;i++) {

					var _error = _errors[i];

					errorContainer.appendChild(this.document.createTextNode(_error.getTestCase()+": "+_error.getTest() +" - "));
					errorContainer.appendChild(me.formatException(_error.getException()));
					errorContainer.appendChild(this.document.createElement('br'));
				}

				errorContainer.style.color = 'red';
				this.container.appendChild(errorTitleContainer);
				this.container.appendChild(errorContainer);
			}

			this.container.appendChild(this.document.createElement("br"));
		}

		// status bar
		var statusBar = this.document.createElement("div");
		var statusText = this.document.createTextNode(_result.wasSuccessful()?"OK!":"NOT OK!");
		statusBar.appendChild(statusText);
		statusBar.style.backgroundColor = _result.wasSuccessful()?"lightgreen":"red";
		statusBar.style.color = 'black';
		statusBar.style.fontWeight = 'bold';
		statusBar.style.textAlign = 'center';
		statusBar.style.marginTop = '1em';
		this.container.appendChild(statusBar);
	},
	startTest : function(_test) {},
	endTest : function(_test) {},
	addSuccess : function(_test) { this.printResult("+"); },
	addFailure : function(_test, _failure) { this.printResult("F"); },
	addError : function(_test, _error) { this.printResult("E"); },
	printResult : function(_char) {

		this.summaryTests.appendChild(this.document.createTextNode(_char));
		if(this.runCounter >= 100) {

			this.runCounter = 1;
			this.summaryTests.appendChild(this.document.createElement("br"));

		} else {

			this.runCounter++;
		}
	}
};
