/**
* @class TestResult
* @constructor
*/
var TestResult = function(){this.init.apply(this, arguments);};
exports.TestResult = TestResult;

TestResult.prototype = {
	/** @ignore */
	init : function() {

		this.runTests = 0;
		this.successes = [];
		this.failures = [];
		this.errors = [];
		this.listeners = [];
	},
	addListener : function(_listener) {

		this.listeners[this.listeners.length] = _listener;
	},
	removeListener : function(_listener) {

		var _temp = [];

		for(var i=0;i<this.listeners.length;i++) {

			if(!this.listeners[i] == _listener) {

				_temp[_temp.length] = _listener;
			}
		}

		this.listeners = _temp;
	},
	runCount : function() {

		return this.runTests;
	},
	successCount : function() {

		return this.successes.length();
	},
	getSuccesses : function() {

		return this.successes;
	},
	addSuccess : function(_test) {

		this.successes[this.successes.length] = _test;

		for(var i=0;i<this.listeners.length;i++) {

			this.listeners[i].addSuccess(_test);
		}
	},
	failureCount : function() {

		return this.failures.length;
	},
	getFailures : function() {

		return this.failures;
	},
	addFailure : function(_test, _testCaseName, _exception) {

		this.failures[this.failures.length] = new TestFailure(_test, _testCaseName, _exception);

		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].addFailure(_test, _testCaseName, _exception);
		}
	},
	errorCount : function() {

		return this.errors.length;
	},
	getErrors : function() {

		return this.errors;
	},
	addError : function(_test, _testCaseName, _exception) {

		this.errors[this.errors.length] = new TestFailure(_test, _testCaseName, _exception);
		
		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].addError(_test, _testCaseName, _exception);
		}
	},
	startTest : function(_test) {

		this.runTests += _test.countTestCases();

		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].startTest(_test);
		}
	},
	endTest : function(_test) {

		for(var i=0;i<this.listeners.length;i++) {
		
			this.listeners[i].endTest(_test);
		}
	},
	wasSuccessful : function() {

		return this.failureCount() == 0 && this.errorCount() == 0;
	},
	run : function(_test) {

		this.startTest(_test);
	
		var _success = true;
		try {

			_test.runBare();

		} catch(_exception) {

			_success = false;

			if(_exception.constructor == AssertionFailedError) {

				this.addFailure(_test, _test.constructor.getName(), _exception);

			} else {

				this.addError(_test, _test.constructor.getName(), _exception);
			}
		}

		if(_success) {

			this.addSuccess(_test);
		}

		this.endTest(_test);
	}
};
