/**
* @class TestSuite
* @constructor
* @param {String} _name
*/
var TestSuite = function(){this.init.apply(this, arguments);};
exports.TestSuite = TestSuite;

TestSuite.prototype = {
	/** @ignore */
	init : function(_name, _test) {

		this.name = _name;
		this.testClass = _test;
		this.tests = [];

		if(_test) {

			this.__getTests__(_test);
		}
	},	
	__getTests__ : function(_test) {

		var parentClass = _test;
		var names = [];
		while(true) {

			var methods = [];
			for(var prop in parentClass.prototype) {

				if(typeof parentClass.prototype[prop] == "function") {

					methods[methods.length] = prop;
				}
			}

			for (var i=0;i<methods.length;i++) {

				if(this.__isTestMethod__(methods[i])) {

					this.__addTestMethod__(methods[i], names, _test);
				}
			}

			parentClass = parentClass.parentclass;
			if(!parentClass) {

				break;
			}
		}
	},
	__isTestMethod__ : function(_method) {

		return _method.indexOf("test") == 0;
	},
	__addTestMethod__ : function(_method, _names, _test) {

		for(var i=0;i<_names.length;i++) {

			if(_names[i] == _method) {

				return;
			}
		}

		_names[_names.length] = _method;

		var test = new _test();
		test.setName(_method);

		this.addTest(test);
	},
	getName : function() {

		return this.name;
	},
	getTestClass : function() {

		return this.testClass;
	},
	countTestCases : function() {

		var count = 0;

		for (var i=0;i<this.tests.length;i++) {

			count += this.tests[i].countTestCases();
		}

		return count;
	},
	run : function(_result) {

		var result = null;
		if(_result) {

			result = _result;

		} else {

			result = new TestResult();
		}

		for (var i=0;i<this.tests.length;i++) {

			this.runTest(this.tests[i], result);
		}
	},
	runTest : function(_test, _result) {

		_test.run(_result);
	},
	addTest : function(_test) {

		this.tests[this.tests.length] = _test;
	},
	addTestSuite : function(_testClass) {

		this.addTest(_testClass.suite());
	},
	testAt : function(_index) {

		return this.tests[_index];
	},
	testCount : function() {

		return this.tests.length;
	},
	getTests : function() {

		return this.tests;
	}

};
